_SPIRV_OP(Nop, 0)
_SPIRV_OP(Undef, 1)
_SPIRV_OP(SourceContinued, 2)
_SPIRV_OP(Source, 3)
_SPIRV_OP(SourceExtension, 4)
_SPIRV_OP(Name, 5)
_SPIRV_OP(MemberName, 6)
_SPIRV_OP(String, 7)
_SPIRV_OP(Line, 8)
_SPIRV_OP(Extension, 10)
_SPIRV_OP(ExtInstImport, 11)
_SPIRV_OP(ExtInst, 12)
_SPIRV_OP(MemoryModel, 14)
_SPIRV_OP(EntryPoint, 15)
_SPIRV_OP(ExecutionMode, 16)
_SPIRV_OP(Capability, 17)
_SPIRV_OP(TypeVoid, 19)
_SPIRV_OP(TypeBool, 20)
_SPIRV_OP(TypeInt, 21)
_SPIRV_OP(TypeFloat, 22)
_SPIRV_OP(TypeVector, 23)
_SPIRV_OP(TypeMatrix, 24)
_SPIRV_OP(TypeImage, 25)
_SPIRV_OP(TypeSampler, 26)
_SPIRV_OP(TypeSampledImage, 27)
_SPIRV_OP(TypeArray, 28)
_SPIRV_OP(TypeRuntimeArray, 29)
_SPIRV_OP(TypeStruct, 30)
_SPIRV_OP(TypeOpaque, 31)
_SPIRV_OP(TypePointer, 32)
_SPIRV_OP(TypeFunction, 33)
_SPIRV_OP(TypeEvent, 34)
_SPIRV_OP(TypeDeviceEvent, 35)
_SPIRV_OP(TypeReserveId, 36)
_SPIRV_OP(TypeQueue, 37)
_SPIRV_OP(TypePipe, 38)
_SPIRV_OP(TypeForwardPointer, 39)
_SPIRV_OP(ConstantTrue, 41)
_SPIRV_OP(ConstantFalse, 42)
_SPIRV_OP(Constant, 43)
_SPIRV_OP(ConstantComposite, 44)
_SPIRV_OP(ConstantSampler, 45)
_SPIRV_OP(ConstantNull, 46)
_SPIRV_OP(SpecConstantTrue, 48)
_SPIRV_OP(SpecConstantFalse, 49)
_SPIRV_OP(SpecConstant, 50)
_SPIRV_OP(SpecConstantComposite, 51)
_SPIRV_OP(SpecConstantOp, 52)
_SPIRV_OP(Function, 54)
_SPIRV_OP(FunctionParameter, 55)
_SPIRV_OP(FunctionEnd, 56)
_SPIRV_OP(FunctionCall, 57)
_SPIRV_OP(Variable, 59)
_SPIRV_OP(ImageTexelPointer, 60)
_SPIRV_OP(Load, 61)
_SPIRV_OP(Store, 62)
_SPIRV_OP(CopyMemory, 63)
_SPIRV_OP(CopyMemorySized, 64)
_SPIRV_OP(AccessChain, 65)
_SPIRV_OP(InBoundsAccessChain, 66)
_SPIRV_OP(PtrAccessChain, 67)
_SPIRV_OP(ArrayLength, 68)
_SPIRV_OP(GenericPtrMemSemantics, 69)
_SPIRV_OP(InBoundsPtrAccessChain, 70)
_SPIRV_OP(Decorate, 71)
_SPIRV_OP(MemberDecorate, 72)
_SPIRV_OP(DecorationGroup, 73)
_SPIRV_OP(GroupDecorate, 74)
_SPIRV_OP(GroupMemberDecorate, 75)
_SPIRV_OP(VectorExtractDynamic, 77)
_SPIRV_OP(VectorInsertDynamic, 78)
_SPIRV_OP(VectorShuffle, 79)
_SPIRV_OP(CompositeConstruct, 80)
_SPIRV_OP(CompositeExtract, 81)
_SPIRV_OP(CompositeInsert, 82)
_SPIRV_OP(CopyObject, 83)
_SPIRV_OP(Transpose, 84)
_SPIRV_OP(SampledImage, 86)
_SPIRV_OP(ImageSampleImplicitLod, 87)
_SPIRV_OP(ImageSampleExplicitLod, 88)
_SPIRV_OP(ImageSampleDrefImplicitLod, 89)
_SPIRV_OP(ImageSampleDrefExplicitLod, 90)
_SPIRV_OP(ImageSampleProjImplicitLod, 91)
_SPIRV_OP(ImageSampleProjExplicitLod, 92)
_SPIRV_OP(ImageSampleProjDrefImplicitLod, 93)
_SPIRV_OP(ImageSampleProjDrefExplicitLod, 94)
_SPIRV_OP(ImageFetch, 95)
_SPIRV_OP(ImageGather, 96)
_SPIRV_OP(ImageDrefGather, 97)
_SPIRV_OP(ImageRead, 98)
_SPIRV_OP(ImageWrite, 99)
_SPIRV_OP(Image, 100)
_SPIRV_OP(ImageQueryFormat, 101)
_SPIRV_OP(ImageQueryOrder, 102)
_SPIRV_OP(ImageQuerySizeLod, 103)
_SPIRV_OP(ImageQuerySize, 104)
_SPIRV_OP(ImageQueryLod, 105)
_SPIRV_OP(ImageQueryLevels, 106)
_SPIRV_OP(ImageQuerySamples, 107)
_SPIRV_OP(ConvertFToU, 109)
_SPIRV_OP(ConvertFToS, 110)
_SPIRV_OP(ConvertSToF, 111)
_SPIRV_OP(ConvertUToF, 112)
_SPIRV_OP(UConvert, 113)
_SPIRV_OP(SConvert, 114)
_SPIRV_OP(FConvert, 115)
_SPIRV_OP(QuantizeToF16, 116)
_SPIRV_OP(ConvertPtrToU, 117)
_SPIRV_OP(SatConvertSToU, 118)
_SPIRV_OP(SatConvertUToS, 119)
_SPIRV_OP(ConvertUToPtr, 120)
_SPIRV_OP(PtrCastToGeneric, 121)
_SPIRV_OP(GenericCastToPtr, 122)
_SPIRV_OP(GenericCastToPtrExplicit, 123)
_SPIRV_OP(Bitcast, 124)
_SPIRV_OP(SNegate, 126)
_SPIRV_OP(FNegate, 127)
_SPIRV_OP(IAdd, 128)
_SPIRV_OP(FAdd, 129)
_SPIRV_OP(ISub, 130)
_SPIRV_OP(FSub, 131)
_SPIRV_OP(IMul, 132)
_SPIRV_OP(FMul, 133)
_SPIRV_OP(UDiv, 134)
_SPIRV_OP(SDiv, 135)
_SPIRV_OP(FDiv, 136)
_SPIRV_OP(UMod, 137)
_SPIRV_OP(SRem, 138)
_SPIRV_OP(SMod, 139)
_SPIRV_OP(FRem, 140)
_SPIRV_OP(FMod, 141)
_SPIRV_OP(VectorTimesScalar, 142)
_SPIRV_OP(MatrixTimesScalar, 143)
_SPIRV_OP(VectorTimesMatrix, 144)
_SPIRV_OP(MatrixTimesVector, 145)
_SPIRV_OP(MatrixTimesMatrix, 146)
_SPIRV_OP(OuterProduct, 147)
_SPIRV_OP(Dot, 148)
_SPIRV_OP(IAddCarry, 149)
_SPIRV_OP(ISubBorrow, 150)
_SPIRV_OP(UMulExtended, 151)
_SPIRV_OP(SMulExtended, 152)
_SPIRV_OP(Any, 154)
_SPIRV_OP(All, 155)
_SPIRV_OP(IsNan, 156)
_SPIRV_OP(IsInf, 157)
_SPIRV_OP(IsFinite, 158)
_SPIRV_OP(IsNormal, 159)
_SPIRV_OP(SignBitSet, 160)
_SPIRV_OP(LessOrGreater, 161)
_SPIRV_OP(Ordered, 162)
_SPIRV_OP(Unordered, 163)
_SPIRV_OP(LogicalEqual, 164)
_SPIRV_OP(LogicalNotEqual, 165)
_SPIRV_OP(LogicalOr, 166)
_SPIRV_OP(LogicalAnd, 167)
_SPIRV_OP(LogicalNot, 168)
_SPIRV_OP(Select, 169)
_SPIRV_OP(IEqual, 170)
_SPIRV_OP(INotEqual, 171)
_SPIRV_OP(UGreaterThan, 172)
_SPIRV_OP(SGreaterThan, 173)
_SPIRV_OP(UGreaterThanEqual, 174)
_SPIRV_OP(SGreaterThanEqual, 175)
_SPIRV_OP(ULessThan, 176)
_SPIRV_OP(SLessThan, 177)
_SPIRV_OP(ULessThanEqual, 178)
_SPIRV_OP(SLessThanEqual, 179)
_SPIRV_OP(FOrdEqual, 180)
_SPIRV_OP(FUnordEqual, 181)
_SPIRV_OP(FOrdNotEqual, 182)
_SPIRV_OP(FUnordNotEqual, 183)
_SPIRV_OP(FOrdLessThan, 184)
_SPIRV_OP(FUnordLessThan, 185)
_SPIRV_OP(FOrdGreaterThan, 186)
_SPIRV_OP(FUnordGreaterThan, 187)
_SPIRV_OP(FOrdLessThanEqual, 188)
_SPIRV_OP(FUnordLessThanEqual, 189)
_SPIRV_OP(FOrdGreaterThanEqual, 190)
_SPIRV_OP(FUnordGreaterThanEqual, 191)
_SPIRV_OP(ShiftRightLogical, 194)
_SPIRV_OP(ShiftRightArithmetic, 195)
_SPIRV_OP(ShiftLeftLogical, 196)
_SPIRV_OP(BitwiseOr, 197)
_SPIRV_OP(BitwiseXor, 198)
_SPIRV_OP(BitwiseAnd, 199)
_SPIRV_OP(Not, 200)
_SPIRV_OP(BitFieldInsert, 201)
_SPIRV_OP(BitFieldSExtract, 202)
_SPIRV_OP(BitFieldUExtract, 203)
_SPIRV_OP(BitReverse, 204)
_SPIRV_OP(BitCount, 205)
_SPIRV_OP(DPdx, 207)
_SPIRV_OP(DPdy, 208)
_SPIRV_OP(Fwidth, 209)
_SPIRV_OP(DPdxFine, 210)
_SPIRV_OP(DPdyFine, 211)
_SPIRV_OP(FwidthFine, 212)
_SPIRV_OP(DPdxCoarse, 213)
_SPIRV_OP(DPdyCoarse, 214)
_SPIRV_OP(FwidthCoarse, 215)
_SPIRV_OP(EmitVertex, 218)
_SPIRV_OP(EndPrimitive, 219)
_SPIRV_OP(EmitStreamVertex, 220)
_SPIRV_OP(EndStreamPrimitive, 221)
_SPIRV_OP(ControlBarrier, 224)
_SPIRV_OP(MemoryBarrier, 225)
_SPIRV_OP(AtomicLoad, 227)
_SPIRV_OP(AtomicStore, 228)
_SPIRV_OP(AtomicExchange, 229)
_SPIRV_OP(AtomicCompareExchange, 230)
_SPIRV_OP(AtomicCompareExchangeWeak, 231)
_SPIRV_OP(AtomicIIncrement, 232)
_SPIRV_OP(AtomicIDecrement, 233)
_SPIRV_OP(AtomicIAdd, 234)
_SPIRV_OP(AtomicISub, 235)
_SPIRV_OP(AtomicSMin, 236)
_SPIRV_OP(AtomicUMin, 237)
_SPIRV_OP(AtomicSMax, 238)
_SPIRV_OP(AtomicUMax, 239)
_SPIRV_OP(AtomicAnd, 240)
_SPIRV_OP(AtomicOr, 241)
_SPIRV_OP(AtomicXor, 242)
_SPIRV_OP(Phi, 245)
_SPIRV_OP(LoopMerge, 246)
_SPIRV_OP(SelectionMerge, 247)
_SPIRV_OP(Label, 248)
_SPIRV_OP(Branch, 249)
_SPIRV_OP(BranchConditional, 250)
_SPIRV_OP(Switch, 251)
_SPIRV_OP(Kill, 252)
_SPIRV_OP(Return, 253)
_SPIRV_OP(ReturnValue, 254)
_SPIRV_OP(Unreachable, 255)
_SPIRV_OP(LifetimeStart, 256)
_SPIRV_OP(LifetimeStop, 257)
_SPIRV_OP(GroupAsyncCopy, 259)
_SPIRV_OP(GroupWaitEvents, 260)
_SPIRV_OP(GroupAll, 261)
_SPIRV_OP(GroupAny, 262)
_SPIRV_OP(GroupBroadcast, 263)
_SPIRV_OP(GroupIAdd, 264)
_SPIRV_OP(GroupFAdd, 265)
_SPIRV_OP(GroupFMin, 266)
_SPIRV_OP(GroupUMin, 267)
_SPIRV_OP(GroupSMin, 268)
_SPIRV_OP(GroupFMax, 269)
_SPIRV_OP(GroupUMax, 270)
_SPIRV_OP(GroupSMax, 271)
_SPIRV_OP(ReadPipe, 274)
_SPIRV_OP(WritePipe, 275)
_SPIRV_OP(ReservedReadPipe, 276)
_SPIRV_OP(ReservedWritePipe, 277)
_SPIRV_OP(ReserveReadPipePackets, 278)
_SPIRV_OP(ReserveWritePipePackets, 279)
_SPIRV_OP(CommitReadPipe, 280)
_SPIRV_OP(CommitWritePipe, 281)
_SPIRV_OP(IsValidReserveId, 282)
_SPIRV_OP(GetNumPipePackets, 283)
_SPIRV_OP(GetMaxPipePackets, 284)
_SPIRV_OP(GroupReserveReadPipePackets, 285)
_SPIRV_OP(GroupReserveWritePipePackets, 286)
_SPIRV_OP(GroupCommitReadPipe, 287)
_SPIRV_OP(GroupCommitWritePipe, 288)
_SPIRV_OP(EnqueueMarker, 291)
_SPIRV_OP(EnqueueKernel, 292)
_SPIRV_OP(GetKernelNDrangeSubGroupCount, 293)
_SPIRV_OP(GetKernelNDrangeMaxSubGroupSize, 294)
_SPIRV_OP(GetKernelWorkGroupSize, 295)
_SPIRV_OP(GetKernelPreferredWorkGroupSizeMultiple, 296)
_SPIRV_OP(RetainEvent, 297)
_SPIRV_OP(ReleaseEvent, 298)
_SPIRV_OP(CreateUserEvent, 299)
_SPIRV_OP(IsValidEvent, 300)
_SPIRV_OP(SetUserEventStatus, 301)
_SPIRV_OP(CaptureEventProfilingInfo, 302)
_SPIRV_OP(GetDefaultQueue, 303)
_SPIRV_OP(BuildNDRange, 304)
_SPIRV_OP(ImageSparseSampleImplicitLod, 305)
_SPIRV_OP(ImageSparseSampleExplicitLod, 306)
_SPIRV_OP(ImageSparseSampleDrefImplicitLod, 307)
_SPIRV_OP(ImageSparseSampleDrefExplicitLod, 308)
_SPIRV_OP(ImageSparseSampleProjImplicitLod, 309)
_SPIRV_OP(ImageSparseSampleProjExplicitLod, 310)
_SPIRV_OP(ImageSparseSampleProjDrefImplicitLod, 311)
_SPIRV_OP(ImageSparseSampleProjDrefExplicitLod, 312)
_SPIRV_OP(ImageSparseFetch, 313)
_SPIRV_OP(ImageSparseGather, 314)
_SPIRV_OP(ImageSparseDrefGather, 315)
_SPIRV_OP(ImageSparseTexelsResident, 316)
_SPIRV_OP(NoLine, 317)
_SPIRV_OP(AtomicFlagTestAndSet, 318)
_SPIRV_OP(AtomicFlagClear, 319)
_SPIRV_OP(SizeOf, 321)
_SPIRV_OP(TypePipeStorage, 322)
_SPIRV_OP(ConstantPipeStorage, 323)
_SPIRV_OP(CreatePipeFromPipeStorage, 324)
_SPIRV_OP(ModuleProcessed, 330)
_SPIRV_OP(ExecutionModeId, 331)
_SPIRV_OP(DecorateId, 332)
_SPIRV_OP(GroupNonUniformElect, 333)
_SPIRV_OP(GroupNonUniformAll, 334)
_SPIRV_OP(GroupNonUniformAny, 335)
_SPIRV_OP(GroupNonUniformAllEqual, 336)
_SPIRV_OP(GroupNonUniformBroadcast, 337)
_SPIRV_OP(GroupNonUniformBroadcastFirst, 338)
_SPIRV_OP(GroupNonUniformBallot, 339)
_SPIRV_OP(GroupNonUniformInverseBallot, 340)
_SPIRV_OP(GroupNonUniformBallotBitExtract, 341)
_SPIRV_OP(GroupNonUniformBallotBitCount, 342)
_SPIRV_OP(GroupNonUniformBallotFindLSB, 343)
_SPIRV_OP(GroupNonUniformBallotFindMSB, 344)
_SPIRV_OP(GroupNonUniformShuffle, 345)
_SPIRV_OP(GroupNonUniformShuffleXor, 346)
_SPIRV_OP(GroupNonUniformShuffleUp, 347)
_SPIRV_OP(GroupNonUniformShuffleDown, 348)
_SPIRV_OP(GroupNonUniformIAdd, 349)
_SPIRV_OP(GroupNonUniformFAdd, 350)
_SPIRV_OP(GroupNonUniformIMul, 351)
_SPIRV_OP(GroupNonUniformFMul, 352)
_SPIRV_OP(GroupNonUniformSMin, 353)
_SPIRV_OP(GroupNonUniformUMin, 354)
_SPIRV_OP(GroupNonUniformFMin, 355)
_SPIRV_OP(GroupNonUniformSMax, 356)
_SPIRV_OP(GroupNonUniformUMax, 357)
_SPIRV_OP(GroupNonUniformFMax, 358)
_SPIRV_OP(GroupNonUniformBitwiseAnd, 359)
_SPIRV_OP(GroupNonUniformBitwiseOr, 360)
_SPIRV_OP(GroupNonUniformBitwiseXor, 361)
_SPIRV_OP(GroupNonUniformLogicalAnd, 362)
_SPIRV_OP(GroupNonUniformLogicalOr, 363)
_SPIRV_OP(GroupNonUniformLogicalXor, 364)
_SPIRV_OP(PtrEqual, 401)
_SPIRV_OP(PtrNotEqual, 402)
_SPIRV_OP(PtrDiff, 403)
_SPIRV_OP(CopyLogical, 400)
_SPIRV_OP(TypeUntypedPointerKHR, 4417)
_SPIRV_OP(UntypedVariableKHR, 4418)
_SPIRV_OP(UntypedAccessChainKHR, 4419)
_SPIRV_OP(UntypedInBoundsAccessChainKHR, 4420)
_SPIRV_OP(UntypedPtrAccessChainKHR, 4423)
_SPIRV_OP(UntypedInBoundsPtrAccessChainKHR, 4424)
_SPIRV_OP(UntypedPrefetchKHR, 4426)
_SPIRV_OP(FmaKHR, 4427)
_SPIRV_OP(GroupNonUniformRotateKHR, 4431)
_SPIRV_OP(SDotKHR, 4450)
_SPIRV_OP(UDotKHR, 4451)
_SPIRV_OP(SUDotKHR, 4452)
_SPIRV_OP(SDotAccSatKHR, 4453)
_SPIRV_OP(UDotAccSatKHR, 4454)
_SPIRV_OP(SUDotAccSatKHR, 4455)
_SPIRV_OP(TypeCooperativeMatrixKHR, 4456)
_SPIRV_OP(CooperativeMatrixLoadKHR, 4457)
_SPIRV_OP(CooperativeMatrixStoreKHR, 4458)
_SPIRV_OP(CooperativeMatrixMulAddKHR, 4459)
_SPIRV_OP(CooperativeMatrixLengthKHR, 4460)
_SPIRV_OP(ReadClockKHR, 5056)
_SPIRV_OP(SubgroupShuffleINTEL, 5571)
_SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
_SPIRV_OP(SubgroupShuffleUpINTEL, 5573)
_SPIRV_OP(SubgroupShuffleXorINTEL, 5574)
_SPIRV_OP(SubgroupBlockReadINTEL, 5575)
_SPIRV_OP(SubgroupBlockWriteINTEL, 5576)
_SPIRV_OP(SubgroupImageBlockReadINTEL, 5577)
_SPIRV_OP(SubgroupImageBlockWriteINTEL, 5578)
_SPIRV_OP(SubgroupImageMediaBlockReadINTEL, 5580)
_SPIRV_OP(SubgroupImageMediaBlockWriteINTEL, 5581)
_SPIRV_OP(ConstantFunctionPointerINTEL, 5600)
_SPIRV_OP(FunctionPointerCallINTEL, 5601)
_SPIRV_OP(AsmTargetINTEL, 5609)
_SPIRV_OP(AsmINTEL, 5610)
_SPIRV_OP(AsmCallINTEL, 5611)
_SPIRV_OP(AtomicFMinEXT, 5614)
_SPIRV_OP(AtomicFMaxEXT, 5615)
_SPIRV_OP(AssumeTrueKHR, 5630)
_SPIRV_OP(ExpectKHR, 5631)
_SPIRV_OP(DecorateString, 5632)
_SPIRV_OP(MemberDecorateString, 5633)
_SPIRV_OP(VmeImageINTEL, 5699)
_SPIRV_OP(TypeVmeImageINTEL, 5700)
_SPIRV_OP(TypeAvcImePayloadINTEL, 5701)
_SPIRV_OP(TypeAvcRefPayloadINTEL, 5702)
_SPIRV_OP(TypeAvcSicPayloadINTEL, 5703)
_SPIRV_OP(TypeAvcMcePayloadINTEL, 5704)
_SPIRV_OP(TypeAvcMceResultINTEL, 5705)
_SPIRV_OP(TypeAvcImeResultINTEL, 5706)
_SPIRV_OP(TypeAvcImeResultSingleReferenceStreamoutINTEL, 5707)
_SPIRV_OP(TypeAvcImeResultDualReferenceStreamoutINTEL, 5708)
_SPIRV_OP(TypeAvcImeSingleReferenceStreaminINTEL, 5709)
_SPIRV_OP(TypeAvcImeDualReferenceStreaminINTEL, 5710)
_SPIRV_OP(TypeAvcRefResultINTEL, 5711)
_SPIRV_OP(TypeAvcSicResultINTEL, 5712)
_SPIRV_OP(SubgroupAvcMceGetDefaultInterBaseMultiReferencePenaltyINTEL, 5713)
_SPIRV_OP(SubgroupAvcMceSetInterBaseMultiReferencePenaltyINTEL, 5714)
_SPIRV_OP(SubgroupAvcMceGetDefaultInterShapePenaltyINTEL, 5715)
_SPIRV_OP(SubgroupAvcMceSetInterShapePenaltyINTEL, 5716)
_SPIRV_OP(SubgroupAvcMceGetDefaultInterDirectionPenaltyINTEL, 5717)
_SPIRV_OP(SubgroupAvcMceSetInterDirectionPenaltyINTEL, 5718)
_SPIRV_OP(SubgroupAvcMceGetDefaultIntraLumaShapePenaltyINTEL, 5719)
_SPIRV_OP(SubgroupAvcMceGetDefaultInterMotionVectorCostTableINTEL, 5720)
_SPIRV_OP(SubgroupAvcMceGetDefaultHighPenaltyCostTableINTEL, 5721)
_SPIRV_OP(SubgroupAvcMceGetDefaultMediumPenaltyCostTableINTEL, 5722)
_SPIRV_OP(SubgroupAvcMceGetDefaultLowPenaltyCostTableINTEL, 5723)
_SPIRV_OP(SubgroupAvcMceSetMotionVectorCostFunctionINTEL, 5724)
_SPIRV_OP(SubgroupAvcMceGetDefaultIntraLumaModePenaltyINTEL, 5725)
_SPIRV_OP(SubgroupAvcMceGetDefaultNonDcLumaIntraPenaltyINTEL, 5726)
_SPIRV_OP(SubgroupAvcMceGetDefaultIntraChromaModeBasePenaltyINTEL, 5727)
_SPIRV_OP(SubgroupAvcMceSetAcOnlyHaarINTEL, 5728)
_SPIRV_OP(SubgroupAvcMceSetSourceInterlacedFieldPolarityINTEL, 5729)
_SPIRV_OP(SubgroupAvcMceSetSingleReferenceInterlacedFieldPolarityINTEL, 5730)
_SPIRV_OP(SubgroupAvcMceSetDualReferenceInterlacedFieldPolaritiesINTEL, 5731)
_SPIRV_OP(SubgroupAvcMceConvertToImePayloadINTEL, 5732)
_SPIRV_OP(SubgroupAvcMceConvertToImeResultINTEL, 5733)
_SPIRV_OP(SubgroupAvcMceConvertToRefPayloadINTEL, 5734)
_SPIRV_OP(SubgroupAvcMceConvertToRefResultINTEL, 5735)
_SPIRV_OP(SubgroupAvcMceConvertToSicPayloadINTEL, 5736)
_SPIRV_OP(SubgroupAvcMceConvertToSicResultINTEL, 5737)
_SPIRV_OP(SubgroupAvcMceGetMotionVectorsINTEL, 5738)
_SPIRV_OP(SubgroupAvcMceGetInterDistortionsINTEL, 5739)
_SPIRV_OP(SubgroupAvcMceGetBestInterDistortionsINTEL, 5740)
_SPIRV_OP(SubgroupAvcMceGetInterMajorShapeINTEL, 5741)
_SPIRV_OP(SubgroupAvcMceGetInterMinorShapeINTEL, 5742)
_SPIRV_OP(SubgroupAvcMceGetInterDirectionsINTEL, 5743)
_SPIRV_OP(SubgroupAvcMceGetInterMotionVectorCountINTEL, 5744)
_SPIRV_OP(SubgroupAvcMceGetInterReferenceIdsINTEL, 5745)
_SPIRV_OP(SubgroupAvcMceGetInterReferenceInterlacedFieldPolaritiesINTEL, 5746)
_SPIRV_OP(SubgroupAvcImeInitializeINTEL, 5747)
_SPIRV_OP(SubgroupAvcImeSetSingleReferenceINTEL, 5748)
_SPIRV_OP(SubgroupAvcImeSetDualReferenceINTEL, 5749)
_SPIRV_OP(SubgroupAvcImeRefWindowSizeINTEL, 5750)
_SPIRV_OP(SubgroupAvcImeAdjustRefOffsetINTEL, 5751)
_SPIRV_OP(SubgroupAvcImeConvertToMcePayloadINTEL, 5752)
_SPIRV_OP(SubgroupAvcImeSetMaxMotionVectorCountINTEL, 5753)
_SPIRV_OP(SubgroupAvcImeSetUnidirectionalMixDisableINTEL, 5754)
_SPIRV_OP(SubgroupAvcImeSetEarlySearchTerminationThresholdINTEL, 5755)
_SPIRV_OP(SubgroupAvcImeSetWeightedSadINTEL, 5756)
_SPIRV_OP(SubgroupAvcImeEvaluateWithSingleReferenceINTEL, 5757)
_SPIRV_OP(SubgroupAvcImeEvaluateWithDualReferenceINTEL, 5758)
_SPIRV_OP(SubgroupAvcImeEvaluateWithSingleReferenceStreaminINTEL, 5759)
_SPIRV_OP(SubgroupAvcImeEvaluateWithDualReferenceStreaminINTEL, 5760)
_SPIRV_OP(SubgroupAvcImeEvaluateWithSingleReferenceStreamoutINTEL, 5761)
_SPIRV_OP(SubgroupAvcImeEvaluateWithDualReferenceStreamoutINTEL, 5762)
_SPIRV_OP(SubgroupAvcImeEvaluateWithSingleReferenceStreaminoutINTEL, 5763)
_SPIRV_OP(SubgroupAvcImeEvaluateWithDualReferenceStreaminoutINTEL, 5764)
_SPIRV_OP(SubgroupAvcImeConvertToMceResultINTEL, 5765)
_SPIRV_OP(SubgroupAvcImeGetSingleReferenceStreaminINTEL, 5766)
_SPIRV_OP(SubgroupAvcImeGetDualReferenceStreaminINTEL, 5767)
_SPIRV_OP(SubgroupAvcImeStripSingleReferenceStreamoutINTEL, 5768)
_SPIRV_OP(SubgroupAvcImeStripDualReferenceStreamoutINTEL, 5769)
_SPIRV_OP(SubgroupAvcImeGetStreamoutSingleReferenceMajorShapeMotionVectorsINTEL,
          5770)
_SPIRV_OP(SubgroupAvcImeGetStreamoutSingleReferenceMajorShapeDistortionsINTEL,
          5771)
_SPIRV_OP(SubgroupAvcImeGetStreamoutSingleReferenceMajorShapeReferenceIdsINTEL,
          5772)
_SPIRV_OP(SubgroupAvcImeGetStreamoutDualReferenceMajorShapeMotionVectorsINTEL,
          5773)
_SPIRV_OP(SubgroupAvcImeGetStreamoutDualReferenceMajorShapeDistortionsINTEL,
          5774)
_SPIRV_OP(SubgroupAvcImeGetStreamoutDualReferenceMajorShapeReferenceIdsINTEL,
          5775)
_SPIRV_OP(SubgroupAvcImeGetBorderReachedINTEL, 5776)
_SPIRV_OP(SubgroupAvcImeGetTruncatedSearchIndicationINTEL, 5777)
_SPIRV_OP(SubgroupAvcImeGetUnidirectionalEarlySearchTerminationINTEL, 5778)
_SPIRV_OP(SubgroupAvcImeGetWeightingPatternMinimumMotionVectorINTEL, 5779)
_SPIRV_OP(SubgroupAvcImeGetWeightingPatternMinimumDistortionINTEL, 5780)
_SPIRV_OP(SubgroupAvcFmeInitializeINTEL, 5781)
_SPIRV_OP(SubgroupAvcBmeInitializeINTEL, 5782)
_SPIRV_OP(SubgroupAvcRefConvertToMcePayloadINTEL, 5783)
_SPIRV_OP(SubgroupAvcRefSetBidirectionalMixDisableINTEL, 5784)
_SPIRV_OP(SubgroupAvcRefSetBilinearFilterEnableINTEL, 5785)
_SPIRV_OP(SubgroupAvcRefEvaluateWithSingleReferenceINTEL, 5786)
_SPIRV_OP(SubgroupAvcRefEvaluateWithDualReferenceINTEL, 5787)
_SPIRV_OP(SubgroupAvcRefEvaluateWithMultiReferenceINTEL, 5788)
_SPIRV_OP(SubgroupAvcRefEvaluateWithMultiReferenceInterlacedINTEL, 5789)
_SPIRV_OP(SubgroupAvcRefConvertToMceResultINTEL, 5790)
_SPIRV_OP(SubgroupAvcSicInitializeINTEL, 5791)
_SPIRV_OP(SubgroupAvcSicConfigureSkcINTEL, 5792)
_SPIRV_OP(SubgroupAvcSicConfigureIpeLumaINTEL, 5793)
_SPIRV_OP(SubgroupAvcSicConfigureIpeLumaChromaINTEL, 5794)
_SPIRV_OP(SubgroupAvcSicGetMotionVectorMaskINTEL, 5795)
_SPIRV_OP(SubgroupAvcSicConvertToMcePayloadINTEL, 5796)
_SPIRV_OP(SubgroupAvcSicSetIntraLumaShapePenaltyINTEL, 5797)
_SPIRV_OP(SubgroupAvcSicSetIntraLumaModeCostFunctionINTEL, 5798)
_SPIRV_OP(SubgroupAvcSicSetIntraChromaModeCostFunctionINTEL, 5799)
_SPIRV_OP(SubgroupAvcSicSetBilinearFilterEnableINTEL, 5800)
_SPIRV_OP(SubgroupAvcSicSetSkcForwardTransformEnableINTEL, 5801)
_SPIRV_OP(SubgroupAvcSicSetBlockBasedRawSkipSadINTEL, 5802)
_SPIRV_OP(SubgroupAvcSicEvaluateIpeINTEL, 5803)
_SPIRV_OP(SubgroupAvcSicEvaluateWithSingleReferenceINTEL, 5804)
_SPIRV_OP(SubgroupAvcSicEvaluateWithDualReferenceINTEL, 5805)
_SPIRV_OP(SubgroupAvcSicEvaluateWithMultiReferenceINTEL, 5806)
_SPIRV_OP(SubgroupAvcSicEvaluateWithMultiReferenceInterlacedINTEL, 5807)
_SPIRV_OP(SubgroupAvcSicConvertToMceResultINTEL, 5808)
_SPIRV_OP(SubgroupAvcSicGetIpeLumaShapeINTEL, 5809)
_SPIRV_OP(SubgroupAvcSicGetBestIpeLumaDistortionINTEL, 5810)
_SPIRV_OP(SubgroupAvcSicGetBestIpeChromaDistortionINTEL, 5811)
_SPIRV_OP(SubgroupAvcSicGetPackedIpeLumaModesINTEL, 5812)
_SPIRV_OP(SubgroupAvcSicGetIpeChromaModeINTEL, 5813)
_SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaCountThresholdINTEL, 5814)
_SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaSumThresholdINTEL, 5815)
_SPIRV_OP(SubgroupAvcSicGetInterRawSadsINTEL, 5816)
_SPIRV_OP(VariableLengthArrayINTEL, 5818)
_SPIRV_OP(SaveMemoryINTEL, 5819)
_SPIRV_OP(RestoreMemoryINTEL, 5820)
_SPIRV_OP(ArbitraryFloatSinCosPiINTEL, 5840)
_SPIRV_OP(ArbitraryFloatCastINTEL, 5841)
_SPIRV_OP(ArbitraryFloatCastFromIntINTEL, 5842)
_SPIRV_OP(ArbitraryFloatCastToIntINTEL, 5843)
_SPIRV_OP(ArbitraryFloatAddINTEL, 5846)
_SPIRV_OP(ArbitraryFloatSubINTEL, 5847)
_SPIRV_OP(ArbitraryFloatMulINTEL, 5848)
_SPIRV_OP(ArbitraryFloatDivINTEL, 5849)
_SPIRV_OP(ArbitraryFloatGTINTEL, 5850)
_SPIRV_OP(ArbitraryFloatGEINTEL, 5851)
_SPIRV_OP(ArbitraryFloatLTINTEL, 5852)
_SPIRV_OP(ArbitraryFloatLEINTEL, 5853)
_SPIRV_OP(ArbitraryFloatEQINTEL, 5854)
_SPIRV_OP(ArbitraryFloatRecipINTEL, 5855)
_SPIRV_OP(ArbitraryFloatRSqrtINTEL, 5856)
_SPIRV_OP(ArbitraryFloatCbrtINTEL, 5857)
_SPIRV_OP(ArbitraryFloatHypotINTEL, 5858)
_SPIRV_OP(ArbitraryFloatSqrtINTEL, 5859)
_SPIRV_OP(ArbitraryFloatLogINTEL, 5860)
_SPIRV_OP(ArbitraryFloatLog2INTEL, 5861)
_SPIRV_OP(ArbitraryFloatLog10INTEL, 5862)
_SPIRV_OP(ArbitraryFloatLog1pINTEL, 5863)
_SPIRV_OP(ArbitraryFloatExpINTEL, 5864)
_SPIRV_OP(ArbitraryFloatExp2INTEL, 5865)
_SPIRV_OP(ArbitraryFloatExp10INTEL, 5866)
_SPIRV_OP(ArbitraryFloatExpm1INTEL, 5867)
_SPIRV_OP(ArbitraryFloatSinINTEL, 5868)
_SPIRV_OP(ArbitraryFloatCosINTEL, 5869)
_SPIRV_OP(ArbitraryFloatSinCosINTEL, 5870)
_SPIRV_OP(ArbitraryFloatSinPiINTEL, 5871)
_SPIRV_OP(ArbitraryFloatCosPiINTEL, 5872)
_SPIRV_OP(ArbitraryFloatASinINTEL, 5873)
_SPIRV_OP(ArbitraryFloatASinPiINTEL, 5874)
_SPIRV_OP(ArbitraryFloatACosINTEL, 5875)
_SPIRV_OP(ArbitraryFloatACosPiINTEL, 5876)
_SPIRV_OP(ArbitraryFloatATanINTEL, 5877)
_SPIRV_OP(ArbitraryFloatATanPiINTEL, 5878)
_SPIRV_OP(ArbitraryFloatATan2INTEL, 5879)
_SPIRV_OP(ArbitraryFloatPowINTEL, 5880)
_SPIRV_OP(ArbitraryFloatPowRINTEL, 5881)
_SPIRV_OP(ArbitraryFloatPowNINTEL, 5882)
_SPIRV_OP(LoopControlINTEL, 5887)
_SPIRV_OP(AliasDomainDeclINTEL, 5911)
_SPIRV_OP(AliasScopeDeclINTEL, 5912)
_SPIRV_OP(AliasScopeListDeclINTEL, 5913)
_SPIRV_OP(FixedSqrtINTEL, 5923)
_SPIRV_OP(FixedRecipINTEL, 5924)
_SPIRV_OP(FixedRsqrtINTEL, 5925)
_SPIRV_OP(FixedSinINTEL, 5926)
_SPIRV_OP(FixedCosINTEL, 5927)
_SPIRV_OP(FixedSinCosINTEL, 5928)
_SPIRV_OP(FixedSinPiINTEL, 5929)
_SPIRV_OP(FixedCosPiINTEL, 5930)
_SPIRV_OP(FixedSinCosPiINTEL, 5931)
_SPIRV_OP(FixedLogINTEL, 5932)
_SPIRV_OP(FixedExpINTEL, 5933)
_SPIRV_OP(PtrCastToCrossWorkgroupINTEL, 5934)
_SPIRV_OP(CrossWorkgroupCastToPtrINTEL, 5938)
_SPIRV_OP(ReadPipeBlockingINTEL, 5946)
_SPIRV_OP(WritePipeBlockingINTEL, 5947)
_SPIRV_OP(FPGARegINTEL, 5949)
_SPIRV_OP(AtomicFAddEXT, 6035)
_SPIRV_OP(TypeBufferSurfaceINTEL, 6086)
_SPIRV_OP(TypeStructContinuedINTEL, 6090)
_SPIRV_OP(ConstantCompositeContinuedINTEL, 6091)
_SPIRV_OP(SpecConstantCompositeContinuedINTEL, 6092)
_SPIRV_OP(ConvertFToBF16INTEL, 6116)
_SPIRV_OP(ConvertBF16ToFINTEL, 6117)
_SPIRV_OP(ControlBarrierArriveINTEL, 6142)
_SPIRV_OP(ControlBarrierWaitINTEL, 6143)
_SPIRV_OP(ArithmeticFenceEXT, 6145)
_SPIRV_OP(Subgroup2DBlockLoadINTEL, 6231)
_SPIRV_OP(Subgroup2DBlockLoadTransformINTEL, 6232)
_SPIRV_OP(Subgroup2DBlockLoadTransposeINTEL, 6233)
_SPIRV_OP(Subgroup2DBlockPrefetchINTEL, 6234)
_SPIRV_OP(Subgroup2DBlockStoreINTEL, 6235)
_SPIRV_OP(SubgroupMatrixMultiplyAccumulateINTEL, 6237)
_SPIRV_OP(BitwiseFunctionINTEL, 6242)
_SPIRV_OP(ConditionalExtensionINTEL, 6248)
_SPIRV_OP(ConditionalEntryPointINTEL, 6249)
_SPIRV_OP(ConditionalCapabilityINTEL, 6250)
_SPIRV_OP(SpecConstantTargetINTEL, 6251)
_SPIRV_OP(SpecConstantArchitectureINTEL, 6252)
_SPIRV_OP(SpecConstantCapabilitiesINTEL, 6253)
_SPIRV_OP(ConditionalCopyObjectINTEL, 6254)
_SPIRV_OP(GroupIMulKHR, 6401)
_SPIRV_OP(GroupFMulKHR, 6402)
_SPIRV_OP(GroupBitwiseAndKHR, 6403)
_SPIRV_OP(GroupBitwiseOrKHR, 6404)
_SPIRV_OP(GroupBitwiseXorKHR, 6405)
_SPIRV_OP(GroupLogicalAndKHR, 6406)
_SPIRV_OP(GroupLogicalOrKHR, 6407)
_SPIRV_OP(GroupLogicalXorKHR, 6408)
_SPIRV_OP(RoundFToTF32INTEL, 6426)
