__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtz(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rte(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtp(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtn(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtz(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rte(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtp(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtn(half);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtz(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rte(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtp(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtn(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtz(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rte(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtp(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtn(float);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtz(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rte(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtp(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_rtn(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtz(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rte(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtp(double);
__attribute__((overloadable)) uchar __spirv_ConvertFToU_Ruchar_sat_rtn(double);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtz(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rte(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtp(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtn(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtz(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rte(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtp(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtn(half2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtz(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rte(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtp(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtn(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtz(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rte(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtp(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtn(float2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtz(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rte(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtp(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_rtn(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtz(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rte(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtp(double2);
__attribute__((overloadable)) uchar2 __spirv_ConvertFToU_Ruchar2_sat_rtn(double2);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtz(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rte(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtp(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtn(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtz(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rte(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtp(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtn(half3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtz(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rte(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtp(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtn(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtz(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rte(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtp(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtn(float3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtz(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rte(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtp(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_rtn(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtz(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rte(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtp(double3);
__attribute__((overloadable)) uchar3 __spirv_ConvertFToU_Ruchar3_sat_rtn(double3);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtz(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rte(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtp(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtn(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtz(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rte(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtp(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtn(half4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtz(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rte(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtp(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtn(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtz(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rte(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtp(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtn(float4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtz(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rte(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtp(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_rtn(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtz(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rte(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtp(double4);
__attribute__((overloadable)) uchar4 __spirv_ConvertFToU_Ruchar4_sat_rtn(double4);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtz(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rte(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtp(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtn(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtz(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rte(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtp(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtn(half8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtz(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rte(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtp(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtn(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtz(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rte(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtp(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtn(float8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtz(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rte(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtp(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_rtn(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtz(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rte(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtp(double8);
__attribute__((overloadable)) uchar8 __spirv_ConvertFToU_Ruchar8_sat_rtn(double8);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtz(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rte(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtp(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtn(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtz(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rte(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtp(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtn(half16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtz(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rte(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtp(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtn(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtz(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rte(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtp(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtn(float16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtz(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rte(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtp(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_rtn(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtz(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rte(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtp(double16);
__attribute__((overloadable)) uchar16 __spirv_ConvertFToU_Ruchar16_sat_rtn(double16);

__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtz(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rte(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtp(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtn(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtz(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rte(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtp(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtn(half);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtz(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rte(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtp(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtn(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtz(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rte(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtp(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtn(float);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtz(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rte(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtp(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_rtn(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtz(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rte(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtp(double);
__attribute__((overloadable)) ushort __spirv_ConvertFToU_Rushort_sat_rtn(double);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtz(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rte(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtp(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtn(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtz(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rte(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtp(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtn(half2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtz(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rte(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtp(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtn(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtz(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rte(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtp(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtn(float2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtz(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rte(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtp(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_rtn(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtz(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rte(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtp(double2);
__attribute__((overloadable)) ushort2 __spirv_ConvertFToU_Rushort2_sat_rtn(double2);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtz(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rte(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtp(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtn(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtz(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rte(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtp(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtn(half3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtz(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rte(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtp(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtn(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtz(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rte(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtp(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtn(float3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtz(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rte(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtp(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_rtn(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtz(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rte(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtp(double3);
__attribute__((overloadable)) ushort3 __spirv_ConvertFToU_Rushort3_sat_rtn(double3);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtz(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rte(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtp(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtn(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtz(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rte(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtp(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtn(half4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtz(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rte(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtp(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtn(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtz(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rte(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtp(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtn(float4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtz(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rte(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtp(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_rtn(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtz(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rte(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtp(double4);
__attribute__((overloadable)) ushort4 __spirv_ConvertFToU_Rushort4_sat_rtn(double4);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtz(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rte(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtp(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtn(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtz(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rte(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtp(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtn(half8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtz(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rte(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtp(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtn(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtz(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rte(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtp(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtn(float8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtz(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rte(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtp(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_rtn(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtz(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rte(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtp(double8);
__attribute__((overloadable)) ushort8 __spirv_ConvertFToU_Rushort8_sat_rtn(double8);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtz(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rte(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtp(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtn(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtz(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rte(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtp(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtn(half16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtz(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rte(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtp(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtn(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtz(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rte(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtp(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtn(float16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtz(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rte(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtp(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_rtn(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtz(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rte(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtp(double16);
__attribute__((overloadable)) ushort16 __spirv_ConvertFToU_Rushort16_sat_rtn(double16);

__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtz(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rte(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtp(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtn(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtz(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rte(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtp(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtn(half);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtz(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rte(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtp(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtn(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtz(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rte(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtp(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtn(float);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtz(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rte(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtp(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_rtn(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtz(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rte(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtp(double);
__attribute__((overloadable)) uint __spirv_ConvertFToU_Ruint_sat_rtn(double);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtz(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rte(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtp(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtn(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtz(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rte(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtp(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtn(half2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtz(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rte(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtp(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtn(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtz(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rte(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtp(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtn(float2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtz(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rte(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtp(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_rtn(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtz(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rte(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtp(double2);
__attribute__((overloadable)) uint2 __spirv_ConvertFToU_Ruint2_sat_rtn(double2);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtz(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rte(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtp(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtn(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtz(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rte(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtp(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtn(half3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtz(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rte(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtp(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtn(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtz(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rte(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtp(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtn(float3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtz(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rte(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtp(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_rtn(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtz(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rte(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtp(double3);
__attribute__((overloadable)) uint3 __spirv_ConvertFToU_Ruint3_sat_rtn(double3);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtz(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rte(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtp(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtn(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtz(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rte(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtp(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtn(half4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtz(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rte(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtp(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtn(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtz(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rte(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtp(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtn(float4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtz(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rte(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtp(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_rtn(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtz(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rte(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtp(double4);
__attribute__((overloadable)) uint4 __spirv_ConvertFToU_Ruint4_sat_rtn(double4);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtz(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rte(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtp(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtn(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtz(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rte(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtp(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtn(half8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtz(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rte(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtp(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtn(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtz(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rte(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtp(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtn(float8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtz(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rte(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtp(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_rtn(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtz(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rte(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtp(double8);
__attribute__((overloadable)) uint8 __spirv_ConvertFToU_Ruint8_sat_rtn(double8);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtz(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rte(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtp(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtn(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtz(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rte(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtp(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtn(half16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtz(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rte(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtp(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtn(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtz(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rte(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtp(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtn(float16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtz(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rte(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtp(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_rtn(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtz(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rte(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtp(double16);
__attribute__((overloadable)) uint16 __spirv_ConvertFToU_Ruint16_sat_rtn(double16);

__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtz(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rte(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtp(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtn(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtz(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rte(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtp(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtn(half);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtz(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rte(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtp(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtn(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtz(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rte(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtp(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtn(float);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtz(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rte(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtp(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_rtn(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtz(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rte(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtp(double);
__attribute__((overloadable)) ulong __spirv_ConvertFToU_Rulong_sat_rtn(double);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtz(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rte(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtp(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtn(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtz(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rte(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtp(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtn(half2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtz(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rte(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtp(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtn(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtz(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rte(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtp(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtn(float2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtz(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rte(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtp(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_rtn(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtz(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rte(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtp(double2);
__attribute__((overloadable)) ulong2 __spirv_ConvertFToU_Rulong2_sat_rtn(double2);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtz(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rte(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtp(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtn(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtz(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rte(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtp(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtn(half3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtz(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rte(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtp(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtn(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtz(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rte(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtp(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtn(float3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtz(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rte(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtp(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_rtn(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtz(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rte(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtp(double3);
__attribute__((overloadable)) ulong3 __spirv_ConvertFToU_Rulong3_sat_rtn(double3);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtz(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rte(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtp(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtn(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtz(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rte(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtp(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtn(half4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtz(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rte(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtp(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtn(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtz(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rte(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtp(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtn(float4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtz(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rte(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtp(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_rtn(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtz(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rte(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtp(double4);
__attribute__((overloadable)) ulong4 __spirv_ConvertFToU_Rulong4_sat_rtn(double4);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtz(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rte(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtp(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtn(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtz(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rte(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtp(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtn(half8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtz(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rte(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtp(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtn(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtz(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rte(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtp(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtn(float8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtz(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rte(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtp(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_rtn(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtz(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rte(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtp(double8);
__attribute__((overloadable)) ulong8 __spirv_ConvertFToU_Rulong8_sat_rtn(double8);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtz(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rte(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtp(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtn(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtz(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rte(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtp(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtn(half16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtz(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rte(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtp(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtn(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtz(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rte(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtp(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtn(float16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtz(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rte(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtp(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_rtn(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtz(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rte(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtp(double16);
__attribute__((overloadable)) ulong16 __spirv_ConvertFToU_Rulong16_sat_rtn(double16);

__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtz(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rte(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtp(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtn(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtz(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rte(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtp(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtn(half);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtz(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rte(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtp(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtn(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtz(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rte(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtp(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtn(float);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtz(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rte(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtp(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_rtn(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtz(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rte(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtp(double);
__attribute__((overloadable)) char __spirv_ConvertFToS_Rchar_sat_rtn(double);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtz(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rte(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtp(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtn(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtz(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rte(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtp(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtn(half2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtz(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rte(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtp(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtn(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtz(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rte(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtp(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtn(float2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtz(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rte(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtp(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_rtn(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtz(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rte(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtp(double2);
__attribute__((overloadable)) char2 __spirv_ConvertFToS_Rchar2_sat_rtn(double2);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtz(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rte(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtp(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtn(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtz(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rte(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtp(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtn(half3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtz(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rte(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtp(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtn(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtz(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rte(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtp(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtn(float3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtz(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rte(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtp(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_rtn(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtz(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rte(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtp(double3);
__attribute__((overloadable)) char3 __spirv_ConvertFToS_Rchar3_sat_rtn(double3);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtz(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rte(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtp(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtn(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtz(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rte(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtp(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtn(half4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtz(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rte(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtp(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtn(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtz(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rte(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtp(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtn(float4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtz(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rte(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtp(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_rtn(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtz(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rte(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtp(double4);
__attribute__((overloadable)) char4 __spirv_ConvertFToS_Rchar4_sat_rtn(double4);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtz(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rte(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtp(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtn(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtz(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rte(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtp(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtn(half8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtz(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rte(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtp(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtn(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtz(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rte(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtp(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtn(float8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtz(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rte(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtp(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_rtn(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtz(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rte(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtp(double8);
__attribute__((overloadable)) char8 __spirv_ConvertFToS_Rchar8_sat_rtn(double8);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtz(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rte(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtp(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtn(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtz(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rte(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtp(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtn(half16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtz(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rte(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtp(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtn(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtz(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rte(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtp(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtn(float16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtz(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rte(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtp(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_rtn(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtz(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rte(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtp(double16);
__attribute__((overloadable)) char16 __spirv_ConvertFToS_Rchar16_sat_rtn(double16);

__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtz(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rte(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtp(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtn(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtz(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rte(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtp(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtn(half);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtz(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rte(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtp(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtn(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtz(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rte(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtp(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtn(float);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtz(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rte(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtp(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_rtn(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtz(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rte(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtp(double);
__attribute__((overloadable)) short __spirv_ConvertFToS_Rshort_sat_rtn(double);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtz(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rte(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtp(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtn(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtz(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rte(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtp(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtn(half2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtz(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rte(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtp(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtn(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtz(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rte(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtp(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtn(float2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtz(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rte(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtp(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_rtn(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtz(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rte(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtp(double2);
__attribute__((overloadable)) short2 __spirv_ConvertFToS_Rshort2_sat_rtn(double2);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtz(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rte(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtp(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtn(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtz(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rte(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtp(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtn(half3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtz(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rte(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtp(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtn(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtz(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rte(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtp(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtn(float3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtz(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rte(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtp(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_rtn(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtz(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rte(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtp(double3);
__attribute__((overloadable)) short3 __spirv_ConvertFToS_Rshort3_sat_rtn(double3);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtz(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rte(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtp(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtn(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtz(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rte(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtp(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtn(half4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtz(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rte(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtp(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtn(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtz(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rte(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtp(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtn(float4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtz(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rte(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtp(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_rtn(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtz(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rte(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtp(double4);
__attribute__((overloadable)) short4 __spirv_ConvertFToS_Rshort4_sat_rtn(double4);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtz(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rte(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtp(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtn(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtz(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rte(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtp(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtn(half8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtz(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rte(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtp(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtn(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtz(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rte(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtp(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtn(float8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtz(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rte(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtp(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_rtn(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtz(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rte(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtp(double8);
__attribute__((overloadable)) short8 __spirv_ConvertFToS_Rshort8_sat_rtn(double8);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtz(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rte(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtp(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtn(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtz(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rte(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtp(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtn(half16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtz(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rte(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtp(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtn(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtz(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rte(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtp(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtn(float16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtz(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rte(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtp(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_rtn(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtz(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rte(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtp(double16);
__attribute__((overloadable)) short16 __spirv_ConvertFToS_Rshort16_sat_rtn(double16);

__attribute__((overloadable)) int __spirv_ConvertFToS_Rint(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtz(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rte(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtp(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtn(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtz(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rte(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtp(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtn(half);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtz(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rte(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtp(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtn(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtz(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rte(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtp(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtn(float);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtz(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rte(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtp(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_rtn(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtz(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rte(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtp(double);
__attribute__((overloadable)) int __spirv_ConvertFToS_Rint_sat_rtn(double);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtz(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rte(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtp(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtn(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtz(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rte(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtp(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtn(half2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtz(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rte(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtp(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtn(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtz(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rte(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtp(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtn(float2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtz(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rte(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtp(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_rtn(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtz(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rte(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtp(double2);
__attribute__((overloadable)) int2 __spirv_ConvertFToS_Rint2_sat_rtn(double2);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtz(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rte(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtp(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtn(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtz(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rte(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtp(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtn(half3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtz(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rte(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtp(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtn(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtz(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rte(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtp(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtn(float3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtz(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rte(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtp(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_rtn(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtz(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rte(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtp(double3);
__attribute__((overloadable)) int3 __spirv_ConvertFToS_Rint3_sat_rtn(double3);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtz(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rte(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtp(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtn(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtz(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rte(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtp(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtn(half4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtz(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rte(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtp(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtn(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtz(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rte(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtp(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtn(float4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtz(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rte(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtp(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_rtn(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtz(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rte(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtp(double4);
__attribute__((overloadable)) int4 __spirv_ConvertFToS_Rint4_sat_rtn(double4);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtz(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rte(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtp(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtn(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtz(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rte(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtp(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtn(half8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtz(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rte(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtp(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtn(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtz(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rte(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtp(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtn(float8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtz(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rte(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtp(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_rtn(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtz(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rte(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtp(double8);
__attribute__((overloadable)) int8 __spirv_ConvertFToS_Rint8_sat_rtn(double8);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtz(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rte(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtp(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtn(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtz(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rte(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtp(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtn(half16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtz(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rte(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtp(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtn(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtz(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rte(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtp(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtn(float16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtz(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rte(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtp(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_rtn(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtz(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rte(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtp(double16);
__attribute__((overloadable)) int16 __spirv_ConvertFToS_Rint16_sat_rtn(double16);

__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtz(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rte(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtp(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtn(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtz(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rte(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtp(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtn(half);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtz(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rte(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtp(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtn(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtz(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rte(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtp(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtn(float);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtz(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rte(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtp(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_rtn(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtz(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rte(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtp(double);
__attribute__((overloadable)) long __spirv_ConvertFToS_Rlong_sat_rtn(double);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtz(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rte(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtp(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtn(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtz(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rte(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtp(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtn(half2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtz(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rte(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtp(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtn(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtz(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rte(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtp(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtn(float2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtz(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rte(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtp(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_rtn(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtz(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rte(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtp(double2);
__attribute__((overloadable)) long2 __spirv_ConvertFToS_Rlong2_sat_rtn(double2);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtz(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rte(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtp(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtn(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtz(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rte(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtp(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtn(half3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtz(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rte(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtp(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtn(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtz(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rte(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtp(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtn(float3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtz(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rte(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtp(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_rtn(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtz(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rte(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtp(double3);
__attribute__((overloadable)) long3 __spirv_ConvertFToS_Rlong3_sat_rtn(double3);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtz(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rte(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtp(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtn(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtz(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rte(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtp(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtn(half4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtz(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rte(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtp(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtn(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtz(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rte(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtp(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtn(float4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtz(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rte(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtp(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_rtn(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtz(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rte(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtp(double4);
__attribute__((overloadable)) long4 __spirv_ConvertFToS_Rlong4_sat_rtn(double4);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtz(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rte(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtp(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtn(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtz(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rte(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtp(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtn(half8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtz(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rte(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtp(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtn(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtz(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rte(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtp(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtn(float8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtz(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rte(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtp(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_rtn(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtz(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rte(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtp(double8);
__attribute__((overloadable)) long8 __spirv_ConvertFToS_Rlong8_sat_rtn(double8);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtz(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rte(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtp(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtn(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtz(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rte(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtp(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtn(half16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtz(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rte(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtp(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtn(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtz(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rte(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtp(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtn(float16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtz(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rte(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtp(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_rtn(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtz(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rte(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtp(double16);
__attribute__((overloadable)) long16 __spirv_ConvertFToS_Rlong16_sat_rtn(double16);

__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf(uchar);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtz(uchar);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rte(uchar);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtp(uchar);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtn(uchar);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf(ushort);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtz(ushort);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rte(ushort);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtp(ushort);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtn(ushort);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf(uint);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtz(uint);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rte(uint);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtp(uint);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtn(uint);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf(ulong);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtz(ulong);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rte(ulong);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtp(ulong);
__attribute__((overloadable)) half __spirv_ConvertUToF_Rhalf_rtn(ulong);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2(uchar2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtz(uchar2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rte(uchar2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtp(uchar2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtn(uchar2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2(ushort2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtz(ushort2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rte(ushort2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtp(ushort2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtn(ushort2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2(uint2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtz(uint2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rte(uint2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtp(uint2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtn(uint2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2(ulong2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtz(ulong2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rte(ulong2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtp(ulong2);
__attribute__((overloadable)) half2 __spirv_ConvertUToF_Rhalf2_rtn(ulong2);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3(uchar3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtz(uchar3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rte(uchar3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtp(uchar3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtn(uchar3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3(ushort3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtz(ushort3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rte(ushort3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtp(ushort3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtn(ushort3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3(uint3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtz(uint3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rte(uint3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtp(uint3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtn(uint3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3(ulong3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtz(ulong3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rte(ulong3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtp(ulong3);
__attribute__((overloadable)) half3 __spirv_ConvertUToF_Rhalf3_rtn(ulong3);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4(uchar4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtz(uchar4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rte(uchar4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtp(uchar4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtn(uchar4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4(ushort4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtz(ushort4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rte(ushort4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtp(ushort4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtn(ushort4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4(uint4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtz(uint4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rte(uint4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtp(uint4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtn(uint4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4(ulong4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtz(ulong4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rte(ulong4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtp(ulong4);
__attribute__((overloadable)) half4 __spirv_ConvertUToF_Rhalf4_rtn(ulong4);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8(uchar8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtz(uchar8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rte(uchar8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtp(uchar8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtn(uchar8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8(ushort8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtz(ushort8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rte(ushort8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtp(ushort8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtn(ushort8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8(uint8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtz(uint8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rte(uint8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtp(uint8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtn(uint8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8(ulong8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtz(ulong8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rte(ulong8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtp(ulong8);
__attribute__((overloadable)) half8 __spirv_ConvertUToF_Rhalf8_rtn(ulong8);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16(uchar16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtz(uchar16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rte(uchar16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtp(uchar16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtn(uchar16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16(ushort16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtz(ushort16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rte(ushort16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtp(ushort16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtn(ushort16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16(uint16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtz(uint16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rte(uint16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtp(uint16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtn(uint16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16(ulong16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtz(ulong16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rte(ulong16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtp(ulong16);
__attribute__((overloadable)) half16 __spirv_ConvertUToF_Rhalf16_rtn(ulong16);

__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat(uchar);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtz(uchar);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rte(uchar);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtp(uchar);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtn(uchar);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat(ushort);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtz(ushort);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rte(ushort);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtp(ushort);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtn(ushort);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat(uint);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtz(uint);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rte(uint);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtp(uint);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtn(uint);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat(ulong);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtz(ulong);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rte(ulong);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtp(ulong);
__attribute__((overloadable)) float __spirv_ConvertUToF_Rfloat_rtn(ulong);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2(uchar2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtz(uchar2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rte(uchar2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtp(uchar2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtn(uchar2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2(ushort2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtz(ushort2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rte(ushort2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtp(ushort2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtn(ushort2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2(uint2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtz(uint2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rte(uint2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtp(uint2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtn(uint2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2(ulong2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtz(ulong2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rte(ulong2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtp(ulong2);
__attribute__((overloadable)) float2 __spirv_ConvertUToF_Rfloat2_rtn(ulong2);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3(uchar3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtz(uchar3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rte(uchar3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtp(uchar3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtn(uchar3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3(ushort3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtz(ushort3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rte(ushort3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtp(ushort3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtn(ushort3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3(uint3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtz(uint3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rte(uint3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtp(uint3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtn(uint3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3(ulong3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtz(ulong3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rte(ulong3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtp(ulong3);
__attribute__((overloadable)) float3 __spirv_ConvertUToF_Rfloat3_rtn(ulong3);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4(uchar4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtz(uchar4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rte(uchar4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtp(uchar4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtn(uchar4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4(ushort4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtz(ushort4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rte(ushort4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtp(ushort4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtn(ushort4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4(uint4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtz(uint4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rte(uint4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtp(uint4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtn(uint4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4(ulong4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtz(ulong4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rte(ulong4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtp(ulong4);
__attribute__((overloadable)) float4 __spirv_ConvertUToF_Rfloat4_rtn(ulong4);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8(uchar8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtz(uchar8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rte(uchar8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtp(uchar8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtn(uchar8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8(ushort8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtz(ushort8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rte(ushort8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtp(ushort8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtn(ushort8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8(uint8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtz(uint8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rte(uint8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtp(uint8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtn(uint8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8(ulong8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtz(ulong8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rte(ulong8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtp(ulong8);
__attribute__((overloadable)) float8 __spirv_ConvertUToF_Rfloat8_rtn(ulong8);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16(uchar16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtz(uchar16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rte(uchar16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtp(uchar16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtn(uchar16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16(ushort16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtz(ushort16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rte(ushort16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtp(ushort16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtn(ushort16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16(uint16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtz(uint16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rte(uint16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtp(uint16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtn(uint16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16(ulong16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtz(ulong16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rte(ulong16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtp(ulong16);
__attribute__((overloadable)) float16 __spirv_ConvertUToF_Rfloat16_rtn(ulong16);

__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble(uchar);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtz(uchar);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rte(uchar);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtp(uchar);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtn(uchar);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble(ushort);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtz(ushort);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rte(ushort);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtp(ushort);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtn(ushort);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble(uint);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtz(uint);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rte(uint);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtp(uint);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtn(uint);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble(ulong);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtz(ulong);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rte(ulong);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtp(ulong);
__attribute__((overloadable)) double __spirv_ConvertUToF_Rdouble_rtn(ulong);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2(uchar2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtz(uchar2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rte(uchar2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtp(uchar2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtn(uchar2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2(ushort2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtz(ushort2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rte(ushort2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtp(ushort2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtn(ushort2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2(uint2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtz(uint2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rte(uint2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtp(uint2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtn(uint2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2(ulong2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtz(ulong2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rte(ulong2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtp(ulong2);
__attribute__((overloadable)) double2 __spirv_ConvertUToF_Rdouble2_rtn(ulong2);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3(uchar3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtz(uchar3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rte(uchar3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtp(uchar3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtn(uchar3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3(ushort3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtz(ushort3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rte(ushort3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtp(ushort3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtn(ushort3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3(uint3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtz(uint3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rte(uint3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtp(uint3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtn(uint3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3(ulong3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtz(ulong3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rte(ulong3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtp(ulong3);
__attribute__((overloadable)) double3 __spirv_ConvertUToF_Rdouble3_rtn(ulong3);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4(uchar4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtz(uchar4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rte(uchar4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtp(uchar4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtn(uchar4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4(ushort4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtz(ushort4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rte(ushort4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtp(ushort4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtn(ushort4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4(uint4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtz(uint4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rte(uint4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtp(uint4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtn(uint4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4(ulong4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtz(ulong4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rte(ulong4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtp(ulong4);
__attribute__((overloadable)) double4 __spirv_ConvertUToF_Rdouble4_rtn(ulong4);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8(uchar8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtz(uchar8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rte(uchar8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtp(uchar8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtn(uchar8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8(ushort8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtz(ushort8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rte(ushort8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtp(ushort8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtn(ushort8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8(uint8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtz(uint8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rte(uint8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtp(uint8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtn(uint8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8(ulong8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtz(ulong8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rte(ulong8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtp(ulong8);
__attribute__((overloadable)) double8 __spirv_ConvertUToF_Rdouble8_rtn(ulong8);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16(uchar16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtz(uchar16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rte(uchar16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtp(uchar16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtn(uchar16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16(ushort16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtz(ushort16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rte(ushort16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtp(ushort16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtn(ushort16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16(uint16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtz(uint16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rte(uint16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtp(uint16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtn(uint16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16(ulong16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtz(ulong16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rte(ulong16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtp(ulong16);
__attribute__((overloadable)) double16 __spirv_ConvertUToF_Rdouble16_rtn(ulong16);

__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf(char);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtz(char);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rte(char);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtp(char);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtn(char);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf(short);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtz(short);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rte(short);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtp(short);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtn(short);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf(int);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtz(int);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rte(int);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtp(int);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtn(int);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf(long);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtz(long);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rte(long);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtp(long);
__attribute__((overloadable)) half __spirv_ConvertSToF_Rhalf_rtn(long);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2(char2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtz(char2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rte(char2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtp(char2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtn(char2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2(short2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtz(short2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rte(short2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtp(short2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtn(short2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2(int2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtz(int2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rte(int2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtp(int2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtn(int2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2(long2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtz(long2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rte(long2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtp(long2);
__attribute__((overloadable)) half2 __spirv_ConvertSToF_Rhalf2_rtn(long2);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3(char3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtz(char3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rte(char3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtp(char3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtn(char3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3(short3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtz(short3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rte(short3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtp(short3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtn(short3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3(int3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtz(int3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rte(int3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtp(int3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtn(int3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3(long3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtz(long3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rte(long3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtp(long3);
__attribute__((overloadable)) half3 __spirv_ConvertSToF_Rhalf3_rtn(long3);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4(char4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtz(char4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rte(char4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtp(char4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtn(char4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4(short4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtz(short4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rte(short4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtp(short4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtn(short4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4(int4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtz(int4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rte(int4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtp(int4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtn(int4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4(long4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtz(long4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rte(long4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtp(long4);
__attribute__((overloadable)) half4 __spirv_ConvertSToF_Rhalf4_rtn(long4);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8(char8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtz(char8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rte(char8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtp(char8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtn(char8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8(short8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtz(short8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rte(short8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtp(short8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtn(short8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8(int8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtz(int8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rte(int8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtp(int8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtn(int8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8(long8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtz(long8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rte(long8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtp(long8);
__attribute__((overloadable)) half8 __spirv_ConvertSToF_Rhalf8_rtn(long8);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16(char16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtz(char16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rte(char16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtp(char16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtn(char16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16(short16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtz(short16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rte(short16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtp(short16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtn(short16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16(int16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtz(int16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rte(int16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtp(int16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtn(int16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16(long16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtz(long16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rte(long16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtp(long16);
__attribute__((overloadable)) half16 __spirv_ConvertSToF_Rhalf16_rtn(long16);

__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat(char);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtz(char);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rte(char);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtp(char);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtn(char);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat(short);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtz(short);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rte(short);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtp(short);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtn(short);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat(int);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtz(int);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rte(int);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtp(int);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtn(int);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat(long);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtz(long);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rte(long);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtp(long);
__attribute__((overloadable)) float __spirv_ConvertSToF_Rfloat_rtn(long);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2(char2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtz(char2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rte(char2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtp(char2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtn(char2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2(short2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtz(short2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rte(short2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtp(short2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtn(short2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2(int2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtz(int2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rte(int2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtp(int2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtn(int2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2(long2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtz(long2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rte(long2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtp(long2);
__attribute__((overloadable)) float2 __spirv_ConvertSToF_Rfloat2_rtn(long2);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3(char3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtz(char3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rte(char3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtp(char3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtn(char3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3(short3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtz(short3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rte(short3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtp(short3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtn(short3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3(int3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtz(int3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rte(int3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtp(int3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtn(int3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3(long3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtz(long3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rte(long3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtp(long3);
__attribute__((overloadable)) float3 __spirv_ConvertSToF_Rfloat3_rtn(long3);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4(char4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtz(char4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rte(char4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtp(char4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtn(char4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4(short4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtz(short4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rte(short4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtp(short4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtn(short4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4(int4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtz(int4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rte(int4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtp(int4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtn(int4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4(long4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtz(long4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rte(long4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtp(long4);
__attribute__((overloadable)) float4 __spirv_ConvertSToF_Rfloat4_rtn(long4);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8(char8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtz(char8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rte(char8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtp(char8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtn(char8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8(short8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtz(short8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rte(short8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtp(short8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtn(short8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8(int8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtz(int8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rte(int8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtp(int8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtn(int8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8(long8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtz(long8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rte(long8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtp(long8);
__attribute__((overloadable)) float8 __spirv_ConvertSToF_Rfloat8_rtn(long8);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16(char16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtz(char16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rte(char16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtp(char16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtn(char16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16(short16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtz(short16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rte(short16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtp(short16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtn(short16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16(int16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtz(int16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rte(int16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtp(int16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtn(int16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16(long16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtz(long16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rte(long16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtp(long16);
__attribute__((overloadable)) float16 __spirv_ConvertSToF_Rfloat16_rtn(long16);

__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble(char);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtz(char);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rte(char);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtp(char);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtn(char);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble(short);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtz(short);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rte(short);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtp(short);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtn(short);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble(int);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtz(int);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rte(int);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtp(int);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtn(int);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble(long);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtz(long);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rte(long);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtp(long);
__attribute__((overloadable)) double __spirv_ConvertSToF_Rdouble_rtn(long);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2(char2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtz(char2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rte(char2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtp(char2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtn(char2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2(short2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtz(short2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rte(short2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtp(short2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtn(short2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2(int2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtz(int2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rte(int2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtp(int2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtn(int2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2(long2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtz(long2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rte(long2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtp(long2);
__attribute__((overloadable)) double2 __spirv_ConvertSToF_Rdouble2_rtn(long2);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3(char3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtz(char3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rte(char3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtp(char3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtn(char3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3(short3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtz(short3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rte(short3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtp(short3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtn(short3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3(int3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtz(int3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rte(int3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtp(int3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtn(int3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3(long3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtz(long3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rte(long3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtp(long3);
__attribute__((overloadable)) double3 __spirv_ConvertSToF_Rdouble3_rtn(long3);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4(char4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtz(char4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rte(char4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtp(char4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtn(char4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4(short4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtz(short4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rte(short4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtp(short4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtn(short4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4(int4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtz(int4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rte(int4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtp(int4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtn(int4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4(long4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtz(long4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rte(long4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtp(long4);
__attribute__((overloadable)) double4 __spirv_ConvertSToF_Rdouble4_rtn(long4);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8(char8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtz(char8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rte(char8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtp(char8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtn(char8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8(short8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtz(short8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rte(short8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtp(short8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtn(short8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8(int8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtz(int8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rte(int8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtp(int8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtn(int8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8(long8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtz(long8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rte(long8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtp(long8);
__attribute__((overloadable)) double8 __spirv_ConvertSToF_Rdouble8_rtn(long8);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16(char16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtz(char16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rte(char16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtp(char16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtn(char16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16(short16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtz(short16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rte(short16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtp(short16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtn(short16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16(int16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtz(int16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rte(int16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtp(int16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtn(int16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16(long16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtz(long16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rte(long16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtp(long16);
__attribute__((overloadable)) double16 __spirv_ConvertSToF_Rdouble16_rtn(long16);

__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtz(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rte(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtp(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtn(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtz(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rte(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtp(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtn(uchar);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtz(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rte(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtp(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtn(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtz(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rte(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtp(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtn(ushort);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtz(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rte(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtp(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtn(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtz(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rte(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtp(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtn(uint);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtz(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rte(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtp(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_rtn(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtz(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rte(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtp(ulong);
__attribute__((overloadable)) uchar __spirv_UConvert_Ruchar_sat_rtn(ulong);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtz(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rte(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtp(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtn(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtz(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rte(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtp(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtn(uchar2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtz(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rte(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtp(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtn(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtz(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rte(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtp(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtn(ushort2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtz(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rte(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtp(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtn(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtz(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rte(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtp(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtn(uint2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtz(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rte(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtp(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_rtn(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtz(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rte(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtp(ulong2);
__attribute__((overloadable)) uchar2 __spirv_UConvert_Ruchar2_sat_rtn(ulong2);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtz(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rte(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtp(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtn(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtz(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rte(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtp(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtn(uchar3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtz(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rte(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtp(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtn(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtz(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rte(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtp(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtn(ushort3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtz(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rte(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtp(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtn(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtz(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rte(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtp(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtn(uint3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtz(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rte(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtp(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_rtn(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtz(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rte(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtp(ulong3);
__attribute__((overloadable)) uchar3 __spirv_UConvert_Ruchar3_sat_rtn(ulong3);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtz(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rte(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtp(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtn(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtz(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rte(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtp(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtn(uchar4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtz(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rte(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtp(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtn(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtz(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rte(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtp(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtn(ushort4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtz(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rte(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtp(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtn(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtz(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rte(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtp(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtn(uint4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtz(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rte(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtp(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_rtn(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtz(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rte(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtp(ulong4);
__attribute__((overloadable)) uchar4 __spirv_UConvert_Ruchar4_sat_rtn(ulong4);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtz(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rte(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtp(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtn(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtz(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rte(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtp(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtn(uchar8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtz(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rte(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtp(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtn(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtz(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rte(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtp(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtn(ushort8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtz(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rte(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtp(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtn(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtz(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rte(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtp(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtn(uint8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtz(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rte(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtp(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_rtn(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtz(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rte(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtp(ulong8);
__attribute__((overloadable)) uchar8 __spirv_UConvert_Ruchar8_sat_rtn(ulong8);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtz(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rte(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtp(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtn(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtz(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rte(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtp(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtn(uchar16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtz(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rte(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtp(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtn(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtz(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rte(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtp(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtn(ushort16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtz(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rte(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtp(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtn(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtz(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rte(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtp(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtn(uint16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtz(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rte(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtp(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_rtn(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtz(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rte(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtp(ulong16);
__attribute__((overloadable)) uchar16 __spirv_UConvert_Ruchar16_sat_rtn(ulong16);

__attribute__((overloadable)) ushort __spirv_UConvert_Rushort(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtz(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rte(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtp(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtn(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtz(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rte(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtp(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtn(uchar);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtz(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rte(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtp(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtn(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtz(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rte(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtp(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtn(ushort);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtz(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rte(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtp(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtn(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtz(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rte(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtp(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtn(uint);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtz(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rte(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtp(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_rtn(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtz(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rte(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtp(ulong);
__attribute__((overloadable)) ushort __spirv_UConvert_Rushort_sat_rtn(ulong);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtz(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rte(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtp(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtn(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtz(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rte(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtp(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtn(uchar2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtz(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rte(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtp(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtn(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtz(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rte(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtp(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtn(ushort2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtz(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rte(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtp(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtn(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtz(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rte(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtp(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtn(uint2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtz(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rte(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtp(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_rtn(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtz(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rte(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtp(ulong2);
__attribute__((overloadable)) ushort2 __spirv_UConvert_Rushort2_sat_rtn(ulong2);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtz(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rte(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtp(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtn(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtz(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rte(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtp(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtn(uchar3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtz(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rte(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtp(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtn(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtz(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rte(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtp(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtn(ushort3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtz(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rte(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtp(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtn(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtz(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rte(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtp(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtn(uint3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtz(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rte(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtp(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_rtn(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtz(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rte(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtp(ulong3);
__attribute__((overloadable)) ushort3 __spirv_UConvert_Rushort3_sat_rtn(ulong3);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtz(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rte(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtp(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtn(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtz(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rte(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtp(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtn(uchar4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtz(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rte(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtp(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtn(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtz(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rte(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtp(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtn(ushort4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtz(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rte(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtp(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtn(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtz(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rte(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtp(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtn(uint4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtz(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rte(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtp(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_rtn(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtz(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rte(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtp(ulong4);
__attribute__((overloadable)) ushort4 __spirv_UConvert_Rushort4_sat_rtn(ulong4);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtz(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rte(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtp(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtn(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtz(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rte(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtp(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtn(uchar8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtz(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rte(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtp(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtn(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtz(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rte(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtp(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtn(ushort8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtz(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rte(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtp(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtn(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtz(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rte(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtp(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtn(uint8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtz(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rte(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtp(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_rtn(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtz(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rte(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtp(ulong8);
__attribute__((overloadable)) ushort8 __spirv_UConvert_Rushort8_sat_rtn(ulong8);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtz(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rte(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtp(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtn(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtz(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rte(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtp(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtn(uchar16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtz(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rte(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtp(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtn(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtz(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rte(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtp(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtn(ushort16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtz(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rte(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtp(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtn(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtz(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rte(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtp(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtn(uint16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtz(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rte(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtp(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_rtn(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtz(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rte(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtp(ulong16);
__attribute__((overloadable)) ushort16 __spirv_UConvert_Rushort16_sat_rtn(ulong16);

__attribute__((overloadable)) uint __spirv_UConvert_Ruint(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtz(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rte(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtp(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtn(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtz(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rte(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtp(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtn(uchar);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtz(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rte(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtp(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtn(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtz(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rte(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtp(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtn(ushort);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtz(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rte(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtp(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtn(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtz(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rte(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtp(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtn(uint);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtz(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rte(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtp(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_rtn(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtz(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rte(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtp(ulong);
__attribute__((overloadable)) uint __spirv_UConvert_Ruint_sat_rtn(ulong);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtz(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rte(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtp(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtn(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtz(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rte(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtp(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtn(uchar2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtz(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rte(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtp(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtn(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtz(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rte(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtp(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtn(ushort2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtz(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rte(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtp(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtn(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtz(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rte(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtp(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtn(uint2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtz(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rte(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtp(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_rtn(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtz(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rte(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtp(ulong2);
__attribute__((overloadable)) uint2 __spirv_UConvert_Ruint2_sat_rtn(ulong2);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtz(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rte(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtp(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtn(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtz(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rte(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtp(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtn(uchar3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtz(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rte(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtp(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtn(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtz(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rte(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtp(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtn(ushort3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtz(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rte(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtp(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtn(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtz(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rte(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtp(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtn(uint3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtz(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rte(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtp(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_rtn(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtz(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rte(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtp(ulong3);
__attribute__((overloadable)) uint3 __spirv_UConvert_Ruint3_sat_rtn(ulong3);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtz(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rte(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtp(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtn(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtz(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rte(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtp(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtn(uchar4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtz(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rte(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtp(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtn(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtz(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rte(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtp(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtn(ushort4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtz(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rte(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtp(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtn(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtz(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rte(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtp(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtn(uint4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtz(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rte(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtp(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_rtn(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtz(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rte(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtp(ulong4);
__attribute__((overloadable)) uint4 __spirv_UConvert_Ruint4_sat_rtn(ulong4);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtz(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rte(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtp(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtn(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtz(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rte(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtp(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtn(uchar8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtz(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rte(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtp(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtn(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtz(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rte(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtp(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtn(ushort8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtz(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rte(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtp(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtn(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtz(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rte(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtp(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtn(uint8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtz(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rte(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtp(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_rtn(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtz(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rte(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtp(ulong8);
__attribute__((overloadable)) uint8 __spirv_UConvert_Ruint8_sat_rtn(ulong8);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtz(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rte(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtp(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtn(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtz(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rte(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtp(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtn(uchar16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtz(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rte(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtp(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtn(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtz(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rte(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtp(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtn(ushort16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtz(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rte(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtp(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtn(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtz(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rte(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtp(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtn(uint16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtz(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rte(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtp(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_rtn(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtz(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rte(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtp(ulong16);
__attribute__((overloadable)) uint16 __spirv_UConvert_Ruint16_sat_rtn(ulong16);

__attribute__((overloadable)) ulong __spirv_UConvert_Rulong(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtz(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rte(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtp(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtn(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtz(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rte(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtp(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtn(uchar);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtz(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rte(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtp(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtn(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtz(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rte(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtp(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtn(ushort);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtz(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rte(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtp(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtn(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtz(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rte(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtp(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtn(uint);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtz(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rte(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtp(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_rtn(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtz(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rte(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtp(ulong);
__attribute__((overloadable)) ulong __spirv_UConvert_Rulong_sat_rtn(ulong);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtz(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rte(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtp(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtn(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtz(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rte(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtp(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtn(uchar2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtz(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rte(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtp(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtn(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtz(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rte(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtp(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtn(ushort2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtz(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rte(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtp(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtn(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtz(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rte(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtp(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtn(uint2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtz(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rte(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtp(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_rtn(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtz(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rte(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtp(ulong2);
__attribute__((overloadable)) ulong2 __spirv_UConvert_Rulong2_sat_rtn(ulong2);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtz(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rte(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtp(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtn(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtz(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rte(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtp(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtn(uchar3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtz(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rte(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtp(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtn(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtz(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rte(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtp(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtn(ushort3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtz(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rte(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtp(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtn(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtz(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rte(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtp(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtn(uint3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtz(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rte(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtp(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_rtn(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtz(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rte(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtp(ulong3);
__attribute__((overloadable)) ulong3 __spirv_UConvert_Rulong3_sat_rtn(ulong3);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtz(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rte(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtp(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtn(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtz(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rte(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtp(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtn(uchar4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtz(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rte(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtp(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtn(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtz(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rte(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtp(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtn(ushort4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtz(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rte(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtp(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtn(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtz(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rte(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtp(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtn(uint4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtz(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rte(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtp(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_rtn(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtz(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rte(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtp(ulong4);
__attribute__((overloadable)) ulong4 __spirv_UConvert_Rulong4_sat_rtn(ulong4);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtz(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rte(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtp(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtn(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtz(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rte(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtp(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtn(uchar8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtz(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rte(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtp(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtn(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtz(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rte(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtp(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtn(ushort8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtz(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rte(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtp(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtn(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtz(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rte(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtp(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtn(uint8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtz(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rte(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtp(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_rtn(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtz(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rte(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtp(ulong8);
__attribute__((overloadable)) ulong8 __spirv_UConvert_Rulong8_sat_rtn(ulong8);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtz(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rte(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtp(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtn(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtz(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rte(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtp(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtn(uchar16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtz(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rte(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtp(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtn(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtz(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rte(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtp(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtn(ushort16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtz(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rte(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtp(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtn(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtz(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rte(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtp(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtn(uint16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtz(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rte(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtp(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_rtn(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtz(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rte(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtp(ulong16);
__attribute__((overloadable)) ulong16 __spirv_UConvert_Rulong16_sat_rtn(ulong16);

__attribute__((overloadable)) char __spirv_SConvert_Rchar(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtz(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rte(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtp(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtn(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtz(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rte(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtp(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtn(char);
__attribute__((overloadable)) char __spirv_SConvert_Rchar(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtz(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rte(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtp(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtn(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtz(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rte(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtp(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtn(short);
__attribute__((overloadable)) char __spirv_SConvert_Rchar(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtz(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rte(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtp(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtn(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtz(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rte(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtp(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtn(int);
__attribute__((overloadable)) char __spirv_SConvert_Rchar(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtz(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rte(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtp(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_rtn(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtz(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rte(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtp(long);
__attribute__((overloadable)) char __spirv_SConvert_Rchar_sat_rtn(long);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtz(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rte(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtp(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtn(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtz(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rte(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtp(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtn(char2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtz(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rte(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtp(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtn(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtz(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rte(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtp(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtn(short2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtz(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rte(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtp(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtn(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtz(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rte(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtp(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtn(int2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtz(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rte(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtp(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_rtn(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtz(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rte(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtp(long2);
__attribute__((overloadable)) char2 __spirv_SConvert_Rchar2_sat_rtn(long2);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtz(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rte(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtp(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtn(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtz(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rte(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtp(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtn(char3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtz(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rte(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtp(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtn(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtz(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rte(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtp(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtn(short3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtz(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rte(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtp(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtn(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtz(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rte(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtp(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtn(int3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtz(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rte(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtp(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_rtn(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtz(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rte(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtp(long3);
__attribute__((overloadable)) char3 __spirv_SConvert_Rchar3_sat_rtn(long3);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtz(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rte(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtp(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtn(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtz(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rte(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtp(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtn(char4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtz(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rte(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtp(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtn(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtz(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rte(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtp(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtn(short4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtz(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rte(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtp(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtn(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtz(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rte(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtp(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtn(int4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtz(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rte(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtp(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_rtn(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtz(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rte(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtp(long4);
__attribute__((overloadable)) char4 __spirv_SConvert_Rchar4_sat_rtn(long4);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtz(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rte(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtp(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtn(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtz(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rte(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtp(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtn(char8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtz(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rte(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtp(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtn(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtz(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rte(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtp(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtn(short8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtz(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rte(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtp(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtn(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtz(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rte(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtp(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtn(int8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtz(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rte(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtp(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_rtn(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtz(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rte(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtp(long8);
__attribute__((overloadable)) char8 __spirv_SConvert_Rchar8_sat_rtn(long8);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtz(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rte(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtp(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtn(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtz(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rte(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtp(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtn(char16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtz(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rte(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtp(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtn(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtz(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rte(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtp(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtn(short16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtz(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rte(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtp(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtn(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtz(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rte(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtp(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtn(int16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtz(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rte(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtp(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_rtn(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtz(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rte(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtp(long16);
__attribute__((overloadable)) char16 __spirv_SConvert_Rchar16_sat_rtn(long16);

__attribute__((overloadable)) short __spirv_SConvert_Rshort(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtz(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rte(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtp(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtn(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtz(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rte(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtp(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtn(char);
__attribute__((overloadable)) short __spirv_SConvert_Rshort(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtz(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rte(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtp(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtn(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtz(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rte(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtp(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtn(short);
__attribute__((overloadable)) short __spirv_SConvert_Rshort(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtz(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rte(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtp(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtn(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtz(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rte(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtp(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtn(int);
__attribute__((overloadable)) short __spirv_SConvert_Rshort(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtz(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rte(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtp(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_rtn(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtz(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rte(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtp(long);
__attribute__((overloadable)) short __spirv_SConvert_Rshort_sat_rtn(long);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtz(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rte(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtp(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtn(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtz(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rte(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtp(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtn(char2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtz(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rte(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtp(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtn(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtz(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rte(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtp(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtn(short2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtz(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rte(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtp(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtn(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtz(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rte(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtp(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtn(int2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtz(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rte(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtp(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_rtn(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtz(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rte(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtp(long2);
__attribute__((overloadable)) short2 __spirv_SConvert_Rshort2_sat_rtn(long2);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtz(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rte(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtp(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtn(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtz(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rte(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtp(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtn(char3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtz(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rte(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtp(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtn(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtz(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rte(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtp(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtn(short3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtz(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rte(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtp(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtn(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtz(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rte(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtp(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtn(int3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtz(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rte(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtp(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_rtn(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtz(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rte(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtp(long3);
__attribute__((overloadable)) short3 __spirv_SConvert_Rshort3_sat_rtn(long3);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtz(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rte(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtp(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtn(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtz(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rte(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtp(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtn(char4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtz(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rte(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtp(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtn(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtz(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rte(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtp(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtn(short4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtz(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rte(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtp(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtn(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtz(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rte(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtp(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtn(int4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtz(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rte(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtp(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_rtn(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtz(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rte(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtp(long4);
__attribute__((overloadable)) short4 __spirv_SConvert_Rshort4_sat_rtn(long4);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtz(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rte(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtp(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtn(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtz(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rte(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtp(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtn(char8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtz(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rte(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtp(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtn(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtz(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rte(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtp(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtn(short8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtz(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rte(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtp(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtn(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtz(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rte(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtp(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtn(int8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtz(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rte(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtp(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_rtn(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtz(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rte(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtp(long8);
__attribute__((overloadable)) short8 __spirv_SConvert_Rshort8_sat_rtn(long8);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtz(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rte(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtp(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtn(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtz(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rte(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtp(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtn(char16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtz(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rte(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtp(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtn(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtz(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rte(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtp(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtn(short16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtz(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rte(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtp(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtn(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtz(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rte(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtp(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtn(int16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtz(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rte(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtp(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_rtn(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtz(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rte(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtp(long16);
__attribute__((overloadable)) short16 __spirv_SConvert_Rshort16_sat_rtn(long16);

__attribute__((overloadable)) int __spirv_SConvert_Rint(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtz(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rte(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtp(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtn(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtz(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rte(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtp(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtn(char);
__attribute__((overloadable)) int __spirv_SConvert_Rint(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtz(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rte(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtp(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtn(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtz(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rte(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtp(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtn(short);
__attribute__((overloadable)) int __spirv_SConvert_Rint(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtz(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rte(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtp(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtn(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtz(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rte(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtp(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtn(int);
__attribute__((overloadable)) int __spirv_SConvert_Rint(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtz(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rte(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtp(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_rtn(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtz(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rte(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtp(long);
__attribute__((overloadable)) int __spirv_SConvert_Rint_sat_rtn(long);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtz(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rte(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtp(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtn(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtz(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rte(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtp(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtn(char2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtz(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rte(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtp(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtn(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtz(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rte(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtp(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtn(short2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtz(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rte(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtp(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtn(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtz(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rte(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtp(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtn(int2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtz(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rte(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtp(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_rtn(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtz(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rte(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtp(long2);
__attribute__((overloadable)) int2 __spirv_SConvert_Rint2_sat_rtn(long2);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtz(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rte(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtp(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtn(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtz(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rte(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtp(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtn(char3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtz(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rte(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtp(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtn(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtz(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rte(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtp(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtn(short3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtz(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rte(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtp(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtn(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtz(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rte(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtp(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtn(int3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtz(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rte(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtp(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_rtn(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtz(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rte(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtp(long3);
__attribute__((overloadable)) int3 __spirv_SConvert_Rint3_sat_rtn(long3);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtz(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rte(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtp(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtn(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtz(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rte(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtp(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtn(char4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtz(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rte(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtp(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtn(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtz(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rte(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtp(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtn(short4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtz(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rte(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtp(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtn(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtz(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rte(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtp(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtn(int4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtz(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rte(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtp(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_rtn(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtz(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rte(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtp(long4);
__attribute__((overloadable)) int4 __spirv_SConvert_Rint4_sat_rtn(long4);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtz(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rte(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtp(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtn(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtz(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rte(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtp(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtn(char8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtz(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rte(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtp(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtn(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtz(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rte(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtp(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtn(short8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtz(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rte(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtp(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtn(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtz(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rte(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtp(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtn(int8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtz(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rte(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtp(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_rtn(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtz(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rte(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtp(long8);
__attribute__((overloadable)) int8 __spirv_SConvert_Rint8_sat_rtn(long8);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtz(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rte(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtp(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtn(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtz(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rte(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtp(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtn(char16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtz(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rte(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtp(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtn(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtz(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rte(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtp(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtn(short16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtz(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rte(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtp(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtn(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtz(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rte(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtp(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtn(int16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtz(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rte(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtp(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_rtn(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtz(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rte(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtp(long16);
__attribute__((overloadable)) int16 __spirv_SConvert_Rint16_sat_rtn(long16);

__attribute__((overloadable)) long __spirv_SConvert_Rlong(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtz(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rte(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtp(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtn(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtz(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rte(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtp(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtn(char);
__attribute__((overloadable)) long __spirv_SConvert_Rlong(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtz(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rte(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtp(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtn(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtz(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rte(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtp(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtn(short);
__attribute__((overloadable)) long __spirv_SConvert_Rlong(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtz(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rte(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtp(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtn(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtz(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rte(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtp(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtn(int);
__attribute__((overloadable)) long __spirv_SConvert_Rlong(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtz(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rte(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtp(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_rtn(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtz(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rte(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtp(long);
__attribute__((overloadable)) long __spirv_SConvert_Rlong_sat_rtn(long);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtz(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rte(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtp(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtn(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtz(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rte(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtp(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtn(char2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtz(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rte(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtp(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtn(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtz(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rte(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtp(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtn(short2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtz(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rte(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtp(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtn(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtz(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rte(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtp(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtn(int2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtz(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rte(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtp(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_rtn(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtz(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rte(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtp(long2);
__attribute__((overloadable)) long2 __spirv_SConvert_Rlong2_sat_rtn(long2);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtz(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rte(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtp(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtn(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtz(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rte(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtp(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtn(char3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtz(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rte(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtp(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtn(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtz(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rte(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtp(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtn(short3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtz(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rte(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtp(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtn(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtz(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rte(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtp(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtn(int3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtz(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rte(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtp(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_rtn(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtz(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rte(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtp(long3);
__attribute__((overloadable)) long3 __spirv_SConvert_Rlong3_sat_rtn(long3);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtz(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rte(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtp(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtn(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtz(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rte(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtp(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtn(char4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtz(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rte(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtp(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtn(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtz(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rte(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtp(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtn(short4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtz(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rte(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtp(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtn(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtz(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rte(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtp(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtn(int4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtz(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rte(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtp(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_rtn(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtz(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rte(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtp(long4);
__attribute__((overloadable)) long4 __spirv_SConvert_Rlong4_sat_rtn(long4);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtz(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rte(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtp(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtn(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtz(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rte(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtp(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtn(char8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtz(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rte(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtp(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtn(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtz(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rte(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtp(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtn(short8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtz(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rte(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtp(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtn(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtz(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rte(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtp(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtn(int8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtz(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rte(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtp(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_rtn(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtz(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rte(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtp(long8);
__attribute__((overloadable)) long8 __spirv_SConvert_Rlong8_sat_rtn(long8);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtz(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rte(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtp(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtn(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtz(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rte(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtp(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtn(char16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtz(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rte(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtp(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtn(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtz(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rte(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtp(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtn(short16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtz(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rte(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtp(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtn(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtz(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rte(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtp(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtn(int16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtz(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rte(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtp(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_rtn(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtz(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rte(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtp(long16);
__attribute__((overloadable)) long16 __spirv_SConvert_Rlong16_sat_rtn(long16);

__attribute__((overloadable)) half __spirv_FConvert_Rhalf(half);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtz(half);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rte(half);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtp(half);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtn(half);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf(float);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtz(float);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rte(float);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtp(float);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtn(float);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf(double);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtz(double);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rte(double);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtp(double);
__attribute__((overloadable)) half __spirv_FConvert_Rhalf_rtn(double);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2(half2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtz(half2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rte(half2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtp(half2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtn(half2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2(float2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtz(float2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rte(float2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtp(float2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtn(float2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2(double2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtz(double2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rte(double2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtp(double2);
__attribute__((overloadable)) half2 __spirv_FConvert_Rhalf2_rtn(double2);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3(half3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtz(half3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rte(half3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtp(half3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtn(half3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3(float3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtz(float3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rte(float3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtp(float3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtn(float3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3(double3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtz(double3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rte(double3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtp(double3);
__attribute__((overloadable)) half3 __spirv_FConvert_Rhalf3_rtn(double3);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4(half4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtz(half4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rte(half4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtp(half4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtn(half4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4(float4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtz(float4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rte(float4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtp(float4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtn(float4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4(double4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtz(double4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rte(double4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtp(double4);
__attribute__((overloadable)) half4 __spirv_FConvert_Rhalf4_rtn(double4);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8(half8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtz(half8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rte(half8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtp(half8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtn(half8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8(float8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtz(float8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rte(float8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtp(float8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtn(float8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8(double8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtz(double8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rte(double8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtp(double8);
__attribute__((overloadable)) half8 __spirv_FConvert_Rhalf8_rtn(double8);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16(half16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtz(half16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rte(half16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtp(half16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtn(half16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16(float16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtz(float16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rte(float16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtp(float16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtn(float16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16(double16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtz(double16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rte(double16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtp(double16);
__attribute__((overloadable)) half16 __spirv_FConvert_Rhalf16_rtn(double16);

__attribute__((overloadable)) float __spirv_FConvert_Rfloat(half);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtz(half);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rte(half);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtp(half);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtn(half);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat(float);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtz(float);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rte(float);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtp(float);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtn(float);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat(double);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtz(double);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rte(double);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtp(double);
__attribute__((overloadable)) float __spirv_FConvert_Rfloat_rtn(double);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2(half2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtz(half2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rte(half2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtp(half2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtn(half2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2(float2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtz(float2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rte(float2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtp(float2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtn(float2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2(double2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtz(double2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rte(double2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtp(double2);
__attribute__((overloadable)) float2 __spirv_FConvert_Rfloat2_rtn(double2);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3(half3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtz(half3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rte(half3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtp(half3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtn(half3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3(float3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtz(float3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rte(float3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtp(float3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtn(float3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3(double3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtz(double3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rte(double3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtp(double3);
__attribute__((overloadable)) float3 __spirv_FConvert_Rfloat3_rtn(double3);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4(half4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtz(half4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rte(half4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtp(half4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtn(half4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4(float4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtz(float4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rte(float4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtp(float4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtn(float4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4(double4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtz(double4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rte(double4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtp(double4);
__attribute__((overloadable)) float4 __spirv_FConvert_Rfloat4_rtn(double4);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8(half8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtz(half8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rte(half8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtp(half8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtn(half8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8(float8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtz(float8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rte(float8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtp(float8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtn(float8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8(double8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtz(double8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rte(double8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtp(double8);
__attribute__((overloadable)) float8 __spirv_FConvert_Rfloat8_rtn(double8);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16(half16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtz(half16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rte(half16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtp(half16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtn(half16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16(float16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtz(float16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rte(float16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtp(float16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtn(float16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16(double16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtz(double16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rte(double16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtp(double16);
__attribute__((overloadable)) float16 __spirv_FConvert_Rfloat16_rtn(double16);

__attribute__((overloadable)) double __spirv_FConvert_Rdouble(half);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtz(half);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rte(half);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtp(half);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtn(half);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble(float);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtz(float);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rte(float);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtp(float);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtn(float);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble(double);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtz(double);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rte(double);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtp(double);
__attribute__((overloadable)) double __spirv_FConvert_Rdouble_rtn(double);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2(half2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtz(half2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rte(half2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtp(half2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtn(half2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2(float2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtz(float2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rte(float2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtp(float2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtn(float2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2(double2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtz(double2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rte(double2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtp(double2);
__attribute__((overloadable)) double2 __spirv_FConvert_Rdouble2_rtn(double2);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3(half3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtz(half3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rte(half3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtp(half3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtn(half3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3(float3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtz(float3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rte(float3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtp(float3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtn(float3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3(double3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtz(double3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rte(double3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtp(double3);
__attribute__((overloadable)) double3 __spirv_FConvert_Rdouble3_rtn(double3);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4(half4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtz(half4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rte(half4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtp(half4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtn(half4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4(float4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtz(float4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rte(float4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtp(float4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtn(float4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4(double4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtz(double4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rte(double4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtp(double4);
__attribute__((overloadable)) double4 __spirv_FConvert_Rdouble4_rtn(double4);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8(half8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtz(half8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rte(half8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtp(half8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtn(half8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8(float8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtz(float8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rte(float8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtp(float8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtn(float8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8(double8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtz(double8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rte(double8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtp(double8);
__attribute__((overloadable)) double8 __spirv_FConvert_Rdouble8_rtn(double8);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16(half16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtz(half16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rte(half16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtp(half16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtn(half16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16(float16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtz(float16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rte(float16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtp(float16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtn(float16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16(double16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtz(double16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rte(double16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtp(double16);
__attribute__((overloadable)) double16 __spirv_FConvert_Rdouble16_rtn(double16);

__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtz(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rte(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtp(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtn(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtz(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rte(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtp(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtn(char);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtz(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rte(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtp(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtn(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtz(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rte(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtp(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtn(short);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtz(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rte(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtp(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtn(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtz(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rte(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtp(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtn(int);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtz(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rte(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtp(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_rtn(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtz(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rte(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtp(long);
__attribute__((overloadable)) uchar __spirv_SatConvertSToU_Ruchar_sat_rtn(long);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtz(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rte(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtp(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtn(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtz(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rte(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtp(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtn(char2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtz(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rte(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtp(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtn(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtz(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rte(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtp(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtn(short2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtz(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rte(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtp(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtn(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtz(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rte(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtp(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtn(int2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtz(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rte(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtp(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_rtn(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtz(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rte(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtp(long2);
__attribute__((overloadable)) uchar2 __spirv_SatConvertSToU_Ruchar2_sat_rtn(long2);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtz(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rte(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtp(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtn(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtz(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rte(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtp(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtn(char3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtz(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rte(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtp(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtn(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtz(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rte(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtp(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtn(short3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtz(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rte(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtp(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtn(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtz(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rte(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtp(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtn(int3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtz(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rte(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtp(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_rtn(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtz(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rte(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtp(long3);
__attribute__((overloadable)) uchar3 __spirv_SatConvertSToU_Ruchar3_sat_rtn(long3);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtz(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rte(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtp(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtn(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtz(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rte(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtp(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtn(char4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtz(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rte(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtp(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtn(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtz(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rte(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtp(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtn(short4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtz(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rte(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtp(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtn(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtz(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rte(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtp(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtn(int4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtz(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rte(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtp(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_rtn(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtz(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rte(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtp(long4);
__attribute__((overloadable)) uchar4 __spirv_SatConvertSToU_Ruchar4_sat_rtn(long4);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtz(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rte(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtp(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtn(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtz(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rte(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtp(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtn(char8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtz(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rte(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtp(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtn(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtz(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rte(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtp(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtn(short8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtz(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rte(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtp(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtn(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtz(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rte(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtp(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtn(int8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtz(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rte(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtp(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_rtn(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtz(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rte(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtp(long8);
__attribute__((overloadable)) uchar8 __spirv_SatConvertSToU_Ruchar8_sat_rtn(long8);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtz(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rte(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtp(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtn(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtz(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rte(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtp(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtn(char16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtz(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rte(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtp(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtn(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtz(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rte(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtp(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtn(short16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtz(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rte(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtp(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtn(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtz(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rte(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtp(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtn(int16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtz(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rte(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtp(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_rtn(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtz(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rte(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtp(long16);
__attribute__((overloadable)) uchar16 __spirv_SatConvertSToU_Ruchar16_sat_rtn(long16);

__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtz(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rte(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtp(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtn(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtz(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rte(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtp(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtn(char);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtz(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rte(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtp(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtn(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtz(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rte(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtp(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtn(short);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtz(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rte(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtp(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtn(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtz(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rte(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtp(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtn(int);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtz(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rte(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtp(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_rtn(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtz(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rte(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtp(long);
__attribute__((overloadable)) ushort __spirv_SatConvertSToU_Rushort_sat_rtn(long);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtz(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rte(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtp(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtn(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtz(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rte(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtp(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtn(char2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtz(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rte(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtp(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtn(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtz(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rte(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtp(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtn(short2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtz(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rte(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtp(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtn(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtz(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rte(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtp(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtn(int2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtz(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rte(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtp(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_rtn(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtz(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rte(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtp(long2);
__attribute__((overloadable)) ushort2 __spirv_SatConvertSToU_Rushort2_sat_rtn(long2);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtz(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rte(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtp(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtn(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtz(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rte(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtp(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtn(char3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtz(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rte(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtp(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtn(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtz(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rte(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtp(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtn(short3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtz(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rte(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtp(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtn(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtz(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rte(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtp(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtn(int3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtz(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rte(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtp(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_rtn(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtz(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rte(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtp(long3);
__attribute__((overloadable)) ushort3 __spirv_SatConvertSToU_Rushort3_sat_rtn(long3);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtz(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rte(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtp(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtn(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtz(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rte(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtp(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtn(char4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtz(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rte(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtp(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtn(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtz(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rte(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtp(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtn(short4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtz(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rte(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtp(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtn(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtz(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rte(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtp(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtn(int4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtz(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rte(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtp(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_rtn(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtz(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rte(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtp(long4);
__attribute__((overloadable)) ushort4 __spirv_SatConvertSToU_Rushort4_sat_rtn(long4);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtz(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rte(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtp(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtn(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtz(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rte(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtp(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtn(char8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtz(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rte(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtp(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtn(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtz(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rte(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtp(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtn(short8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtz(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rte(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtp(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtn(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtz(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rte(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtp(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtn(int8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtz(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rte(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtp(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_rtn(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtz(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rte(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtp(long8);
__attribute__((overloadable)) ushort8 __spirv_SatConvertSToU_Rushort8_sat_rtn(long8);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtz(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rte(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtp(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtn(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtz(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rte(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtp(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtn(char16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtz(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rte(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtp(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtn(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtz(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rte(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtp(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtn(short16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtz(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rte(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtp(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtn(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtz(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rte(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtp(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtn(int16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtz(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rte(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtp(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_rtn(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtz(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rte(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtp(long16);
__attribute__((overloadable)) ushort16 __spirv_SatConvertSToU_Rushort16_sat_rtn(long16);

__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtz(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rte(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtp(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtn(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtz(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rte(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtp(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtn(char);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtz(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rte(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtp(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtn(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtz(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rte(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtp(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtn(short);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtz(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rte(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtp(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtn(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtz(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rte(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtp(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtn(int);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtz(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rte(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtp(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_rtn(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtz(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rte(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtp(long);
__attribute__((overloadable)) uint __spirv_SatConvertSToU_Ruint_sat_rtn(long);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtz(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rte(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtp(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtn(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtz(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rte(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtp(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtn(char2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtz(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rte(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtp(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtn(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtz(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rte(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtp(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtn(short2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtz(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rte(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtp(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtn(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtz(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rte(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtp(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtn(int2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtz(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rte(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtp(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_rtn(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtz(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rte(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtp(long2);
__attribute__((overloadable)) uint2 __spirv_SatConvertSToU_Ruint2_sat_rtn(long2);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtz(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rte(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtp(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtn(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtz(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rte(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtp(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtn(char3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtz(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rte(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtp(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtn(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtz(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rte(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtp(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtn(short3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtz(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rte(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtp(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtn(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtz(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rte(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtp(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtn(int3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtz(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rte(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtp(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_rtn(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtz(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rte(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtp(long3);
__attribute__((overloadable)) uint3 __spirv_SatConvertSToU_Ruint3_sat_rtn(long3);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtz(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rte(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtp(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtn(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtz(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rte(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtp(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtn(char4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtz(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rte(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtp(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtn(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtz(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rte(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtp(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtn(short4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtz(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rte(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtp(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtn(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtz(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rte(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtp(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtn(int4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtz(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rte(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtp(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_rtn(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtz(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rte(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtp(long4);
__attribute__((overloadable)) uint4 __spirv_SatConvertSToU_Ruint4_sat_rtn(long4);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtz(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rte(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtp(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtn(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtz(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rte(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtp(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtn(char8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtz(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rte(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtp(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtn(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtz(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rte(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtp(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtn(short8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtz(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rte(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtp(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtn(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtz(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rte(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtp(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtn(int8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtz(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rte(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtp(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_rtn(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtz(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rte(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtp(long8);
__attribute__((overloadable)) uint8 __spirv_SatConvertSToU_Ruint8_sat_rtn(long8);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtz(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rte(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtp(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtn(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtz(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rte(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtp(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtn(char16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtz(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rte(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtp(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtn(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtz(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rte(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtp(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtn(short16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtz(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rte(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtp(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtn(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtz(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rte(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtp(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtn(int16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtz(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rte(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtp(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_rtn(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtz(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rte(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtp(long16);
__attribute__((overloadable)) uint16 __spirv_SatConvertSToU_Ruint16_sat_rtn(long16);

__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtz(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rte(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtp(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtn(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtz(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rte(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtp(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtn(char);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtz(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rte(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtp(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtn(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtz(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rte(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtp(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtn(short);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtz(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rte(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtp(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtn(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtz(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rte(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtp(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtn(int);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtz(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rte(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtp(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_rtn(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtz(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rte(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtp(long);
__attribute__((overloadable)) ulong __spirv_SatConvertSToU_Rulong_sat_rtn(long);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtz(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rte(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtp(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtn(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtz(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rte(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtp(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtn(char2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtz(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rte(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtp(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtn(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtz(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rte(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtp(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtn(short2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtz(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rte(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtp(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtn(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtz(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rte(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtp(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtn(int2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtz(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rte(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtp(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_rtn(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtz(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rte(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtp(long2);
__attribute__((overloadable)) ulong2 __spirv_SatConvertSToU_Rulong2_sat_rtn(long2);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtz(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rte(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtp(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtn(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtz(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rte(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtp(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtn(char3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtz(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rte(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtp(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtn(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtz(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rte(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtp(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtn(short3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtz(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rte(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtp(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtn(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtz(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rte(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtp(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtn(int3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtz(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rte(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtp(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_rtn(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtz(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rte(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtp(long3);
__attribute__((overloadable)) ulong3 __spirv_SatConvertSToU_Rulong3_sat_rtn(long3);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtz(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rte(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtp(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtn(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtz(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rte(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtp(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtn(char4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtz(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rte(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtp(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtn(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtz(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rte(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtp(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtn(short4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtz(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rte(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtp(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtn(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtz(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rte(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtp(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtn(int4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtz(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rte(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtp(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_rtn(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtz(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rte(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtp(long4);
__attribute__((overloadable)) ulong4 __spirv_SatConvertSToU_Rulong4_sat_rtn(long4);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtz(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rte(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtp(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtn(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtz(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rte(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtp(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtn(char8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtz(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rte(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtp(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtn(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtz(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rte(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtp(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtn(short8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtz(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rte(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtp(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtn(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtz(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rte(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtp(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtn(int8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtz(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rte(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtp(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_rtn(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtz(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rte(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtp(long8);
__attribute__((overloadable)) ulong8 __spirv_SatConvertSToU_Rulong8_sat_rtn(long8);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtz(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rte(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtp(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtn(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtz(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rte(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtp(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtn(char16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtz(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rte(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtp(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtn(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtz(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rte(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtp(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtn(short16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtz(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rte(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtp(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtn(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtz(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rte(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtp(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtn(int16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtz(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rte(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtp(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_rtn(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtz(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rte(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtp(long16);
__attribute__((overloadable)) ulong16 __spirv_SatConvertSToU_Rulong16_sat_rtn(long16);

__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtz(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rte(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtp(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtn(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtz(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rte(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtp(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtn(uchar);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtz(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rte(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtp(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtn(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtz(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rte(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtp(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtn(ushort);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtz(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rte(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtp(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtn(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtz(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rte(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtp(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtn(uint);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtz(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rte(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtp(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_rtn(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtz(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rte(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtp(ulong);
__attribute__((overloadable)) char __spirv_SatConvertUToS_Rchar_sat_rtn(ulong);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtz(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rte(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtp(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtn(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtz(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rte(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtp(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtn(uchar2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtz(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rte(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtp(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtn(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtz(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rte(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtp(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtn(ushort2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtz(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rte(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtp(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtn(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtz(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rte(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtp(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtn(uint2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtz(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rte(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtp(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_rtn(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtz(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rte(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtp(ulong2);
__attribute__((overloadable)) char2 __spirv_SatConvertUToS_Rchar2_sat_rtn(ulong2);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtz(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rte(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtp(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtn(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtz(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rte(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtp(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtn(uchar3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtz(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rte(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtp(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtn(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtz(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rte(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtp(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtn(ushort3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtz(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rte(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtp(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtn(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtz(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rte(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtp(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtn(uint3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtz(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rte(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtp(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_rtn(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtz(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rte(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtp(ulong3);
__attribute__((overloadable)) char3 __spirv_SatConvertUToS_Rchar3_sat_rtn(ulong3);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtz(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rte(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtp(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtn(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtz(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rte(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtp(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtn(uchar4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtz(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rte(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtp(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtn(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtz(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rte(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtp(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtn(ushort4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtz(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rte(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtp(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtn(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtz(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rte(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtp(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtn(uint4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtz(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rte(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtp(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_rtn(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtz(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rte(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtp(ulong4);
__attribute__((overloadable)) char4 __spirv_SatConvertUToS_Rchar4_sat_rtn(ulong4);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtz(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rte(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtp(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtn(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtz(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rte(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtp(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtn(uchar8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtz(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rte(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtp(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtn(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtz(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rte(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtp(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtn(ushort8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtz(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rte(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtp(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtn(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtz(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rte(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtp(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtn(uint8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtz(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rte(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtp(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_rtn(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtz(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rte(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtp(ulong8);
__attribute__((overloadable)) char8 __spirv_SatConvertUToS_Rchar8_sat_rtn(ulong8);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtz(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rte(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtp(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtn(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtz(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rte(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtp(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtn(uchar16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtz(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rte(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtp(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtn(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtz(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rte(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtp(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtn(ushort16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtz(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rte(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtp(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtn(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtz(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rte(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtp(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtn(uint16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtz(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rte(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtp(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_rtn(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtz(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rte(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtp(ulong16);
__attribute__((overloadable)) char16 __spirv_SatConvertUToS_Rchar16_sat_rtn(ulong16);

__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtz(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rte(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtp(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtn(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtz(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rte(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtp(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtn(uchar);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtz(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rte(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtp(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtn(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtz(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rte(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtp(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtn(ushort);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtz(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rte(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtp(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtn(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtz(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rte(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtp(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtn(uint);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtz(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rte(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtp(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_rtn(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtz(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rte(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtp(ulong);
__attribute__((overloadable)) short __spirv_SatConvertUToS_Rshort_sat_rtn(ulong);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtz(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rte(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtp(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtn(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtz(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rte(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtp(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtn(uchar2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtz(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rte(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtp(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtn(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtz(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rte(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtp(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtn(ushort2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtz(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rte(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtp(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtn(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtz(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rte(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtp(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtn(uint2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtz(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rte(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtp(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_rtn(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtz(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rte(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtp(ulong2);
__attribute__((overloadable)) short2 __spirv_SatConvertUToS_Rshort2_sat_rtn(ulong2);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtz(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rte(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtp(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtn(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtz(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rte(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtp(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtn(uchar3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtz(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rte(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtp(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtn(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtz(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rte(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtp(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtn(ushort3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtz(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rte(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtp(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtn(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtz(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rte(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtp(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtn(uint3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtz(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rte(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtp(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_rtn(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtz(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rte(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtp(ulong3);
__attribute__((overloadable)) short3 __spirv_SatConvertUToS_Rshort3_sat_rtn(ulong3);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtz(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rte(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtp(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtn(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtz(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rte(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtp(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtn(uchar4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtz(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rte(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtp(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtn(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtz(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rte(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtp(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtn(ushort4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtz(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rte(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtp(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtn(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtz(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rte(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtp(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtn(uint4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtz(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rte(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtp(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_rtn(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtz(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rte(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtp(ulong4);
__attribute__((overloadable)) short4 __spirv_SatConvertUToS_Rshort4_sat_rtn(ulong4);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtz(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rte(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtp(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtn(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtz(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rte(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtp(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtn(uchar8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtz(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rte(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtp(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtn(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtz(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rte(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtp(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtn(ushort8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtz(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rte(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtp(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtn(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtz(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rte(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtp(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtn(uint8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtz(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rte(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtp(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_rtn(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtz(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rte(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtp(ulong8);
__attribute__((overloadable)) short8 __spirv_SatConvertUToS_Rshort8_sat_rtn(ulong8);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtz(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rte(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtp(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtn(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtz(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rte(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtp(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtn(uchar16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtz(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rte(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtp(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtn(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtz(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rte(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtp(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtn(ushort16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtz(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rte(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtp(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtn(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtz(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rte(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtp(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtn(uint16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtz(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rte(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtp(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_rtn(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtz(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rte(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtp(ulong16);
__attribute__((overloadable)) short16 __spirv_SatConvertUToS_Rshort16_sat_rtn(ulong16);

__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtz(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rte(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtp(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtn(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtz(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rte(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtp(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtn(uchar);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtz(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rte(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtp(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtn(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtz(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rte(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtp(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtn(ushort);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtz(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rte(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtp(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtn(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtz(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rte(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtp(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtn(uint);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtz(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rte(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtp(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_rtn(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtz(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rte(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtp(ulong);
__attribute__((overloadable)) int __spirv_SatConvertUToS_Rint_sat_rtn(ulong);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtz(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rte(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtp(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtn(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtz(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rte(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtp(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtn(uchar2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtz(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rte(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtp(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtn(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtz(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rte(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtp(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtn(ushort2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtz(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rte(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtp(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtn(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtz(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rte(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtp(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtn(uint2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtz(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rte(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtp(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_rtn(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtz(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rte(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtp(ulong2);
__attribute__((overloadable)) int2 __spirv_SatConvertUToS_Rint2_sat_rtn(ulong2);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtz(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rte(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtp(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtn(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtz(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rte(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtp(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtn(uchar3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtz(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rte(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtp(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtn(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtz(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rte(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtp(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtn(ushort3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtz(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rte(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtp(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtn(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtz(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rte(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtp(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtn(uint3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtz(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rte(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtp(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_rtn(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtz(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rte(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtp(ulong3);
__attribute__((overloadable)) int3 __spirv_SatConvertUToS_Rint3_sat_rtn(ulong3);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtz(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rte(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtp(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtn(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtz(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rte(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtp(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtn(uchar4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtz(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rte(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtp(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtn(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtz(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rte(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtp(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtn(ushort4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtz(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rte(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtp(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtn(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtz(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rte(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtp(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtn(uint4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtz(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rte(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtp(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_rtn(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtz(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rte(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtp(ulong4);
__attribute__((overloadable)) int4 __spirv_SatConvertUToS_Rint4_sat_rtn(ulong4);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtz(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rte(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtp(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtn(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtz(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rte(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtp(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtn(uchar8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtz(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rte(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtp(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtn(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtz(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rte(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtp(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtn(ushort8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtz(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rte(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtp(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtn(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtz(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rte(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtp(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtn(uint8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtz(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rte(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtp(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_rtn(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtz(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rte(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtp(ulong8);
__attribute__((overloadable)) int8 __spirv_SatConvertUToS_Rint8_sat_rtn(ulong8);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtz(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rte(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtp(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtn(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtz(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rte(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtp(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtn(uchar16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtz(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rte(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtp(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtn(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtz(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rte(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtp(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtn(ushort16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtz(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rte(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtp(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtn(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtz(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rte(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtp(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtn(uint16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtz(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rte(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtp(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_rtn(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtz(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rte(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtp(ulong16);
__attribute__((overloadable)) int16 __spirv_SatConvertUToS_Rint16_sat_rtn(ulong16);

__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtz(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rte(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtp(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtn(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtz(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rte(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtp(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtn(uchar);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtz(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rte(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtp(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtn(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtz(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rte(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtp(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtn(ushort);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtz(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rte(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtp(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtn(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtz(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rte(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtp(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtn(uint);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtz(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rte(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtp(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_rtn(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtz(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rte(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtp(ulong);
__attribute__((overloadable)) long __spirv_SatConvertUToS_Rlong_sat_rtn(ulong);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtz(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rte(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtp(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtn(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtz(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rte(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtp(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtn(uchar2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtz(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rte(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtp(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtn(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtz(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rte(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtp(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtn(ushort2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtz(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rte(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtp(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtn(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtz(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rte(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtp(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtn(uint2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtz(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rte(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtp(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_rtn(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtz(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rte(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtp(ulong2);
__attribute__((overloadable)) long2 __spirv_SatConvertUToS_Rlong2_sat_rtn(ulong2);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtz(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rte(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtp(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtn(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtz(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rte(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtp(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtn(uchar3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtz(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rte(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtp(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtn(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtz(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rte(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtp(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtn(ushort3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtz(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rte(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtp(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtn(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtz(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rte(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtp(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtn(uint3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtz(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rte(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtp(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_rtn(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtz(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rte(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtp(ulong3);
__attribute__((overloadable)) long3 __spirv_SatConvertUToS_Rlong3_sat_rtn(ulong3);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtz(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rte(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtp(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtn(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtz(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rte(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtp(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtn(uchar4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtz(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rte(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtp(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtn(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtz(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rte(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtp(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtn(ushort4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtz(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rte(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtp(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtn(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtz(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rte(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtp(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtn(uint4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtz(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rte(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtp(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_rtn(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtz(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rte(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtp(ulong4);
__attribute__((overloadable)) long4 __spirv_SatConvertUToS_Rlong4_sat_rtn(ulong4);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtz(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rte(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtp(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtn(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtz(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rte(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtp(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtn(uchar8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtz(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rte(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtp(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtn(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtz(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rte(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtp(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtn(ushort8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtz(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rte(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtp(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtn(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtz(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rte(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtp(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtn(uint8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtz(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rte(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtp(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_rtn(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtz(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rte(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtp(ulong8);
__attribute__((overloadable)) long8 __spirv_SatConvertUToS_Rlong8_sat_rtn(ulong8);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtz(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rte(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtp(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtn(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtz(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rte(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtp(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtn(uchar16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtz(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rte(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtp(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtn(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtz(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rte(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtp(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtn(ushort16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtz(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rte(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtp(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtn(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtz(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rte(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtp(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtn(uint16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtz(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rte(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtp(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_rtn(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtz(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rte(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtp(ulong16);
__attribute__((overloadable)) long16 __spirv_SatConvertUToS_Rlong16_sat_rtn(ulong16);

